C#	$Id: pQCDAdler.f,v 1.4 1998/12/30 12:51:23 fjeger Exp fjeger $	
C;;;;;;;;;;;;;;;;;;;;;;;;;;;; -*- Mode: Fortran -*- ;;;;;;;;;;;;;;;;;;;;;;;;;;;
C;; pQCDAdler.f --
C;; Copyright (C) F. Jegerlehner 1998. Email: <fjeger@ifh.de>
C;;
C;; Author          : Fred Jegerlehner in collaboration with Oleg Veretin
C;; Created On      : Wed Oct  7 19:34:55 1998
C;; Last Modified By: Friedrich Jegerlehner
C;; Last Modified On: Wed Feb 20 15:00:43 2013
C;; Update Count    : 206
C;; Status          : Unknown, Use with caution!
C;;
C;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
c  S.~Eidelman, F.~Jegerlehner, A.~L.~Kataev and O.~Veretin,
c  "Testing nonperturbative strong interaction effects via the Adler function",
c  Phys. Lett. B 454 (1999) 369  [hep-ph/9812521] and referencees therein
c
c makes use of BG-MOM RG; gauge parameter fixed by merging with MSbar in high energy limit
c  F.~Jegerlehner and O.~V.~Tarasov,
c  "Exact mass dependent two loop alpha_s(Q**2) in the background MOM renormalization scheme",
c  Nucl. Phys. B 549 (1999) 481 [hep-ph/9809485]  and referencees therein
c
C Changes: 26/5/2001 : ali(1)=alinp  added (was missing! -> error)
C                      added files pQCDAdler.dap52, pQCDAdler.dam52
C                      for theoretical uncertainties: flag is ierr=3
C                      for ierr=1 writing theoretical uncertainies from OPE 
C Changes: 15/8/2006 : updated QCD parameters PDG Yao JPG 33 (2006) 1;
C                      alpha_s from S. Bethke June 2006
C                      mc,mb from Monahar and Sachrajda in PDG loc cit.
C          08/04/2008 : mc and mb from Sturm, Khn, Steinhauser 2007
C          02/06/2008 : added 5--loops by Baikov, Chetyrkin, Khn 2001
C          26/08/2008 : APT analytic coupling 'anals' implemented 
c                       anals from Lambda values via subroutine analsEanax(e,Lam,anals)
c provides APT couplings of order 1,2 and 3      
c                       anals from data table via subroutine analsEdat(e,anals)
c provides APT couplings of order 1,2 and 3 using interpolating table
c from Aug 2008 actual papers on APT from Dubna group, a preliminary study of personal interests
c
c list of functions and subroutines
c    function f1(x,eps)                               --- one-loop coefficient
c    function f2(x,eps,i)                             --- two-loop coefficient
c    subroutine dthree(x,i,dpiv0,dpiv1,dpiv2)         --- three-loop coefficient
c    subroutine dthreelow(x,i,dPiv00,dPiv10,dPiv20)    -  low energy expansion
c    subroutine dthreePad(x,i)                         -  Pad resummed ~ exact
c    subroutine mstoosmass(mdm,i,adpatmu,rmstoos)     --- on-shell mass from MSbar mass
c    subroutine ostomsmass(mdm,i,adpatmu,rostoms)     --- MSbar mass from on-shell mass
c    subroutine alphaQCD(s,alsMOM,alsMSb,ior)         --- calculate alpha_s in BF-MOM and MSbar 
c    subroutine getnfa(e,nf)                          --- get number od active flavors at scale e
c    subroutine analsEanax(e,Lam,anals)               --- analytic couplings of order 1,2,3
c    subroutine analsEdat(e,anals)                    --- analytic couplings of order 1,2,3 using table
c in dhl.f
c    function DHL(Q2,mm,mu2)                          --- contribution to D(Q^2) from heavylight digarams
c    function heavylight(x,mmmu,approx)               --- contribution to D(Q^2) from heavylight digarams
c
      program pQCDAdler
c calculates Adler function D(Q^2) = H_1 + H_2 (alpha_s/pi) + H_3 (alpha_s/pi)^2 + ...
c 4-- and 5--loop in massless approximation (only improves high energy tail)
c optional: + condensate contribution vis OPE 
C FILES TO BE LINKED: qcdpar2,qcdpar4,vapoqcd2,ddilog,dli3,dhl
      implicit none
      CHARACTER*1 yesno
      integer i,j,n,np,nm,nl,nf,ie,ierr,ianal,ii
      logical test
      INTEGER NFX,FTQ,IOR,isc
      character *1 cfn(4)
      real *8 pi,pix,pi2,ln2,zeta2,zeta3,zeta4,zeta5,b4,i3
      real *8 CF,CA,TF,cd2,cd2x,cd3,cd3x,cd4,cd4x,cd5,cd5x
      real *8 D,E,EP,EM,Q,Q2,x,y,f1,f2,alinp,einp,als,adp,adp2,adp3,adp4
      real *8 alerr,alerf,alsMOM,alsMSb,ali(3),eps,crad
      real *8 d1,d2,d3,e1,e2,dpiv0,dpiv1,dpiv2
      real *8 qq(6),MTOP,ALFAMOM0,ALFAMOM1,ALF1,ALINP2,EINP2
      real *8 lnx,lnx2,z,z1,z2,z3,z4,z5,z6,z7,heas,lqm,fac,sfac,adpj
      real *8 r1,r2,r3,f1ser,f2ser,f2pad,f3ser,f3pad
      real *8 Dnp(3),cgc,cqc,cqk,mqqs(3),agg4(6,3),mqq4(6,3)
      real *8 Emax,Emin,SMIN,SMAX,LOSU,LOSL,DLST,LSLT
      real *8 R0(6),Qr(6),D4,D5,c20,c21,c22,xi,rmstoos,anals(3)
      real *8 DHL,Dadd,h2,h2add,dhl1
      real *8 be0(6),be1(6),be0f,nf_3,beta0_3,beta1_3,a,LAM,LAM2,ln1,l2
      COMMON /PARAM/ALINP2,EINP2,als,adp
      common /fser/f1ser,f2ser,f2pad,f3ser,f3pad,eps,crad
      common /numer/pix,pi2,ln2,zeta2,zeta3,zeta4,zeta5,b4,i3
      common /group/CF,CA,TF,nL
      common /flag/test
      common /rgcoe/be0,be1
      data ln2/0.6931471805599453d0/
      data zeta2,zeta3/1.644934066848226d0,1.20205690315959d0/
      data cfn /'0','1','2','3'/
C Input: Quark masses MQ and physical thresholds TH for switching NF from rqcd-param.f
      integer IER
      double precision pi1,ALINP1,EINP1,MTOP1,ERALS
      DOUBLE PRECISION MQ(6),MP(6),TH(6),dmp(6,3)
      common /inp/pi1,ALINP1,EINP1,MTOP1
      COMMON /QUARK/MQ,MP,TH
      common /QUAER/ERALS,IER
      common /QUAME/DMP
      
      call rqcdparam()          ! fill common inp,QUARK etc; updated April 2008 and 2012
c now as interface to constants_qcd
      pi   =pi1
      alinp=ALINP1
      einp =EINP1
      alerr=ERALS
      ALINP2=ALINP1
      EINP2=EINP1 
      pix  =pi1
*
*     .. Executable Statements ..
*
      nf_3=3.d0
      beta0_3=1.d0/(4.d0*pi)*(11.d0-2.d0/3.d0*nf_3)
      beta1_3=1.d0/(4.d0*pi)**2*(102.d0-38.d0/3.d0*nf_3)
      do ii=1,6 
         be0(ii)=1.d0/(4.d0*pi)*(11.d0-2.d0/3.d0*ii)
         be1(ii)=1.d0/(4.d0*pi)**2*(102.d0-38.d0/3.d0*ii)
      enddo
C Warning: input parameter arrays overwritten by input from screen or via input data file
C pQCDAdler.inp0,1,2
C 30.12.98 added 4-loop
* R values
      R0(2)=5.d0/3.d0
      R0(3)=2.d0
      R0(4)=10.d0/3.d0
      R0(5)=11.d0/3.d0
      R0(6)=15.d0/3.d0
      Qr(2)=1.d0/15.d0
      Qr(3)=0.d0
      Qr(4)=2.d0/15.d0
      Qr(5)=1.d0/33.d0
      Qr(6)=3.d0/15.d0
* data for condensates
      do i=1,3
        agg4(i,1)=(0.389d0)**4   ! Peris,Perrottet,De Rafael (0.08+/-0.02)/pi
        agg4(3+i,1)=0.d0
        mqq4(i,1)=-(0.099d0)**4  ! <qq>=-(0.237+/-0.027)^3; (mu+md)/2=4+/-2
        mqq4(3+i,1)=0.d0
        agg4(i,2)=(0.442d0)**4
        agg4(3+i,2)=0.d0
        mqq4(i,2)=-(0.086d0)**4
        mqq4(3+i,2)=0.d0
        agg4(i,3)=(0.336d0)**4
        agg4(3+i,3)=0.d0
        mqq4(i,3)=-(0.111d0)**4
        mqq4(3+i,3)=0.d0
      enddo
      mqq4(3,1)=-(0.219d0)**4       ! <qq>=-(0.237+/-0.027)^3; ms=175+/-25
      mqq4(3,2)=-(0.192d0)**4
      mqq4(3,3)=-(0.245d0)**4
      do j=1,3
         mqqs(j)=mqq4(1,j)+mqq4(2,j)+mqq4(3,j)
      enddo
* numerical constants
      pi1=pi
      pi2=pi*pi
      zeta4=pi2**2/90.d0
      zeta5=1.03693d0
      B4 =-1.762800d0
      I3 = 3.d0/2.d0*zeta3-6.d0*zeta2*ln2
C check R(s) -> D(Q^2): c_2(D)=c_2(R)+pi^2*beta_0^2/3/16; beta_0=11-2/3*nf
C Gorishny, Kataev, Larin PLB 259 (1991) 144
C         c20=-6.6368d0+pi2/3.d0/16.d0*11.d0**2          ! const
C         c21=-1.2002d0-pi2/3.d0/16.d0*11.d0*4d0/3.d0    ! nf
C         c22=-0.0052d0+pi2/3.d0/16.d0*4.d0/9.d0         ! nf**2
C         write (*,*) ' c20=',c20
C         write (*,*) ' c21=',c21
C         write (*,*) ' c22=',c22
* group factors
      Cf=4.d0/3.d0
      Ca=3.d0
      Tf=0.5d0
      nl=4
      nf=5
      fac=3.d0/4.d0 ! conversion factor for Adler function
* 3--loop high energy asymptotic value
      lqm=0.d0     ! ln(-q^2/mu^2) choosen MSb scale mu^2=Q^2=-q^2
      heas = (((32.d0*CF*NL+32.d0*CF)*TF-88.d0*CA*CF)*ZETA3
     1     +((8.d0*CF*LQM-44.d0*CF)*NL+8.d0*CF*LQM-44.d0*CF)*TF
     1   -22.d0*CA*CF*LQM-3.d0*CF**2+123.d0*CA*CF)/24.d0*fac
      write (*,*) ' heas 3-loop:',heas
* error analysis theoretical uncertainties of pQCD
      ierr=3
c* error analysis theoretical uncertainties of NP OPE
c      ierr=1
* test print flag
      test=.FALSE.
C     test=.TRUE.
* quark masses
      mp(1)=0.1d0
      mp(2)=0.1d0
c      MTOP=MP(6)
* quark charges
      qq(1)=2.d0/3.d0
      qq(2)=-1.d0/3.d0
      qq(3)=qq(2)
      qq(4)=qq(1)
      qq(5)=qq(2)
      qq(6)=qq(1)
* alpha-s input
C update S. Bethke June 2006 and 2012
      alinp1=alinp
      einp1 =einp 
c      alerr= 0.001d0
c      alinp= 0.12d0
c      einp =91.19d0
c      alerr= 0.003d0
c      alinpcorr=alinp/0.12d0  ! 0.12 was used for calculating alphaMOM data files
C                                        which should be updated
* INPUT:
      include 'Input.h'
c worst combination of mass and coupling uncertainty
c      alinpcorr=alinp/0.12d0  ! 0.12 was used for calculating alphaMOM data files
      ali(1)=alinp
      ali(2)=alinp+alerr
      ali(3)=alinp-alerr
c      write (*,*) ' ali:',ali
c anticorrelated combination of mass and coupling uncertainty
c      ali(2)=alinp-alerr
c      ali(3)=alinp+alerr
      crad=.25d0  ! convergence radius for Pad'e: y=4mm/QQ < crad
      eps=1d-6    ! exact -> series expansion switch
      FTQ=0
c      IOR=0
* OUTPUT files and plot headers:
      include 'FilesHeaders.h'
c
      ianal=0             ! flag for use of APT
      write (*,*) ' ianal:',ianal
      Emax=100.d0
      Emin=0.00001d0
c      Emin=0.40000d0
      if (ior.eq.0) Emin=0.00001d0      
      SMIN=EMIN**2
      SMAX=EMAX**2
      LOSU=LOG(SMAX)
      LOSL=LOG(SMIN)
      DLST=(LOSU-LOSL)/(NP-1)
c! ie loop uncertainties
      do ie=1,ierr              ! theoretical uncertainty alinp+/-alerr
c! n  loop energy points
      do n=1,np                 ! main energy loop for plot data
        LSLT=LOSU-(N-1)*DLST
        Q2=EXP(LSLT)
        Q=SQRT(Q2)
C QCD PARAMETERS
C alpha_QCD in MOM BF scheme: ALFAMOM0 uses MSb value as initial data
C ALFAMOM1 the corresponding MOM value at EINP; ALF1 is the MSb coupling
C at E=SQRT(S) calculated from ALFAMOM1.
C      CALL ALQCD3(Q2,ALINP,EINP,ALFAMOM0,ALFAMOM1,ALF1,IOR+1)
C      CALL ALQCD2(Q2,ALINP,EINP,MTOP,ALS,NFX,FTQ,IOR+1)
C using alpha-s in rescaling improved BF-MOM scheme
      if (ior.ge.1) then
        isc =1 ! isc=1,2 for MOM,MSb scheme
        call alphaQCD(Q2,alsMOM,alsMSb,ior+1)
        if (isc.eq.1) then
          als=alsMOM*ali(ie)/alinp
        else if (isc.eq.2) then
          als=alsMSb*ali(ie)/alinp
        else
           als=alinp
        endif
      else
         als=alinp
      endif
c      write (*,*) ior,-Q,als
      lqm=0.d0     ! ln(-q^2/mu^2) choosen MSb scale mu^2=Q^2=-q^2
      adp=als/pi
      adp2=adp*adp
      adp3=adp*adp2
      adp4=adp*adp3
      call getnfa(q,nfx)
      if (ie.eq.1) write (14,9) q,adp,adp2,adp3
      if (ianal.ge.1)  then
         be0f=be0(nfx)
         als=als/(1.d0+be0f*als)
         adp=als/pi
         adp2=adp*adp
         adp3=adp*adp2
         adp4=adp*adp3
         if (ie.eq.1) write (15,9) q,adp,adp2,adp3
      endif
      if (ianal.eq.2)  then
c         LAM=350.d-3
c         LAM=400.d-3
         LAM=214.7d-3
         call analsEanax(q,Lam,anals)
         adp =anals(1)/pi
         adp2=anals(2)/pi2
         adp3=anals(3)/pi2/pi
      endif
      if (ianal.eq.3)  then
         a=4.d0
         LAM=350.d-3
         LAM2=LAM*LAM
         ln1=log(Q2/LAM2)
         l2=ln1+beta1_3/2.d0/beta0_3**2*log(ln1**2+a*pi2)
         adp=1.d0/beta0_3*(1.d0/l2+1.d0/(1.d0-exp(l2)))/pi
         adp2=1.d0/beta0_3**2*
     &        (1.d0/l2**2-exp(l2)/(exp(l2)-1.d0)**2)/pi2
         adp3=1.d0/beta0_3**3*(1.d0/l2**3
     &        -0.5d0*(exp(l2)+exp(2.d0*l2))/(exp(l2)-1.d0)**3)/pi2/pi
      endif
      if (ie.eq.1)  write (16,9)  q,adp,adp2,adp3
c coefficient for condesate OPE
      cgc=8.d0/12.d0*(1.d0-11.d0/18.d0*adp)*pi2
      cqc=8.d0*2.d0*(1.d0+adp/3.d0+(47.d0/8.d0-3.d0/4.d0*lqm)*adp2)*pi2
      cqk=8.d0*(4.d0/27.d0*adp
     &   +(4.d0/3.d0*zeta3-88.d0/243.d0-1.d0/3.d0*lqm)*adp2)*pi2
c      
c test energy dependent (via alpha_s) condensate coefficients
c      write (*,*) ' test np params: cgc,cqc,cqk:',cgc,cqc,cqk
c      
      D1=0.d0
      D2=0.d0
      D3=0.d0
      D4=0.d0
      D5=0.d0
      E1=0.d0
      E2=0.d0
      Dnp(1)=0.d0
      Dnp(2)=0.d0
      Dnp(3)=0.d0
      h2=0.d0
c      
C first calculate pole mass from m_q(m_q)
C     x=mp(i)**2/Q**2
c      do j=4,6
c         xi=1.d0
c         nf=j 
c         call alphaQCD(mq(j)**2,alsMOM,alsMSb,3)
c         adpj=alsMSb/pi
c         call mstoosmass(xi,nf,adpj,rmstoos)
cC     x=x*rmstoos**2
c         write (43,*) j,alsMSb,mq(j),rmstoos
c      enddo
c! i  loop flavor index
      do i=1,6
C non-perturbative contribution
      do j=1,3
        Dnp(j)=Dnp(j)+3.d0*qq(i)**2*
     &         (cgc*agg4(i,j)+cqc*mqq4(i,j)+cqk*mqqs(j))/Q2**2
      enddo
        x=(mp(i)+dmp(i,ie))**2/Q2
        y=4.d0*x
        if (x.lt.0.25d0) then
           call dthree(x,i,dpiv0,dpiv1,dpiv2)
           f3ser=dpiv2
        else if(x.gt.0.25d0) then
           call dthreelow(x,i,dpiv0,dpiv1,dpiv2)
           f3ser=dpiv2
        else
           dpiv0=0.d0
           dpiv1=0.d0
           dpiv2=0.d0
           f3ser=dpiv2
        endif
        if (y.le.crad) then           ! high Q^2 (light quark)
           f3pad=f3ser
        else
           call dthreePad(x,i)
        endif
        r1=f1(x,eps)
        r2=f2(x,eps,i)
        r3=dpiv2
        D1=D1+3.d0*qq(i)**2*r1
        E1=E1+3.d0*qq(i)**2*dpiv0
        D2=D2+3.d0*qq(i)**2*adp*r2
        E2=E2+3.d0*qq(i)**2*adp*dpiv1

c>>

C  contribution from the heavy-light diagrams
C  (insertion of heavy quark loop into light quark loop)
          if ( i .LT. 1 ) then
          f3pad  = f3pad  +  dhl( Q2, mq(1)**2, Q2 )
          if ( i .LT. 2 ) then
          f3pad  =  f3pad  +  dhl( Q2, mq(2)**2, Q2 )
          if ( i .LT. 3 ) then
          f3pad  =  f3pad  +  dhl( Q2, mq(3)**2, Q2 )
          if ( i .LT. 4 ) then
          f3pad  =  f3pad  +  dhl( Q2, mq(4)**2, Q2 )
          if ( i .LT. 5 ) then
          f3pad  =  f3pad  +  dhl( Q2, mq(5)**2, Q2 )
          endif
          endif
          endif
          endif
          endif
C>>
        D3=D3+3.d0*qq(i)**2*adp2*f3pad
c        print*,'Q=',sqrt(Q2),'i=',i,'H2=',h2,h2add
C 27.08.08 scheme without Landau pole implemented
c D/D0=d1+d2*adp*d3*adp2+d4*adp3+....
c     adp4*(be0**3*d2*pi**3+3*be0**2*d3*pi**2+3*be0*d4*pi+d5)
c    +adp3*(be0**2*d2*pi**2+2*be0*d3*pi+d4)
c    +adp2*(be0*d2*pi+d3)
c    +adp*d2
c    +d1
c normalized to d1=1
c scheme dependence starts at D3 : d3 --> d3+pi*beta0*d2
        cd2=(D2/D1)/adp
        cd2x=cd2
        D2=cd2x*adp*D1
        cd3=(D3/D1)/adp2
        if (ianal.eq.1) then
           cd3x=cd3+be0f*cd2*pi
c        if ((ianal.eq.2).and.(q.le.2.6d0)) cd3x=cd3
c        if (ianal.eq.2) cd3x=cd3
        else 
           cd3x=cd3
        endif
        D3=cd3x*adp2*D1
c *************************
c improving high energy tails 4- and 5-loop massless contributions
c
C 30.12.98 added 4-loop
C
C        D4=(-6.6368d0-1.2002d0*nfx-0.0052d0*nfx**2-1.2395d0*Qr(nfx))
C     &     *adp3*R0(nfx)
c scheme dependence d4 --> be0**2*d2*pi**2+2*be0*d3*pi+d4
        cd4=(18.2428d0-4.2159d0*nfx+0.0862d0*nfx**2-1.2395d0*Qr(nfx))
        if (ianal.ge.1) then
           cd4x=cd4+be0f**2*cd2*pi**2+2.d0*be0f*cd3*pi
        else 
           cd4x=cd4
        endif
        D4=cd4x*adp3*R0(nfx)
C 02.06.08 added 5-loop Baikov, Chetyrkin, Khn 2001
c scheme dependence d5 --> be0**3*d2*pi**3+3*be0**2*d3*pi**2+3*be0*d4*pi+d5
        cd5=(-0.010d0*nfx**3+1.88d0*nfx**2-34.4d0*nfx+135.8d0)
        if (ianal.ge.1) then
           cd5x=cd5+be0f**3*cd2*pi**3+3.d0*be0f**2*cd3*pi**2
     &          +3.d0*be0f*cd4*pi
        else 
           cd5x=cd5
        endif
        D5=cd5x*adp4*R0(nfx)
c
c *************************
c
        if (ior.eq.0) then
          D =D1
          E =E1
        else if (ior.eq.1) then
          D =D1+D2
          E =E1+E2
        else
c          D =D1+D2+D3
c          E =D1+D2+D3+Dnp(1)
c          EP=D1+D2+D3+Dnp(2)
c          EM=D1+D2+D3+Dnp(3)
C FJ 16 Aug 2006; FJ 02 June 2008
c           if (ianal.ne.0) then
              D =D1+D2+D3
              E =D1+D2+D3
              EP=D1+D2+D3
              EM=D1+D2+D3
c           else 
              D =D1+D2+D3+D4+D5
              E =D1+D2+D3+D4+D5+Dnp(1)
              EP=D1+D2+D3+D4+D5+Dnp(2)
              EM=D1+D2+D3+D4+D5+Dnp(3)
          write (*,9) -Q,D1+D3,D,Dnp(1)
c           endif
        endif
        if (i.eq.3) then
          write (1,9) -Q,D,E
        else if (i.eq.4) then
          write (2,9) -Q,D,E
        else if (i.eq.5) then
            if (ierr.gt.1) then
              if (ie.eq.1) then
                write (3,9) -Q,D,E
              else if (ie.eq.2) then
                write (8,9) -Q,D
              else if (ie.eq.3) then
                write (9,9) -Q,D
              endif
            else
              write (3,9) -Q,D,E
              write (10,9) -Q,D+D4+D5,D4,D5
            endif
          if ((ior.eq.2).and.(ie.eq.1)) then
            if (ierr.gt.1) then
            else
C OPE condensate extrema
              write (8,9) -Q,D,EP
              write (9,9) -Q,D,EM
            endif
          endif
          if (test) then
* test block begin
             write (11,9) -Q,r1,f1ser,dpiv1
             write (12,9) -Q,r2,f2ser,f2pad
             write (13,9) -Q,r3,r3,f3pad
* test block end
          endif
        else if (i.eq.6) then
          write (4,9) -Q,D,E
        else
        endif
      enddo            ! i  loop flavor index
      enddo            ! n  loop energy points
      enddo            ! ie loop uncertainties
c      write (*,*) ' E:',q,LOSL,EMIN,EXP(LOSL)
 9    format(1x,1pe11.4,3(2x,1pe11.4))
 103  format (1X,a17,3(1x,f9.3))
      stop
      end

      function f1(x,eps)
c calculate one-loop coefficient H_1
      implicit none
      real *8 f1,x,x2,y,rt,xi,fac,z,z1,z2,z3,z4,z5,z6,z7,eps
      real *8 lqm,lox
      real *8 f1ser,f2ser,f2pad,f3ser,f3pad,eps1,crad
      common /fser/f1ser,f2ser,f2pad,f3ser,f3pad,eps1,crad
C     x=mp(i)**2/Q**2
      y=4.d0*x
      x2=x*x
      fac=3.d0/4.d0
* f1 normalized to go to 1 for large Q^2
      if (y.le.1.d0) then
         xi=1.d0/x
         z=-x                       ! z=mp(i)**2/q**2
         z1=z
         z2=z*z
         z3=z2*z
         z4=z3*z
         z5=z4*z
         z6=z5*z
C assume Q^2=-q^2=mu^2 together with alpha_s(Q^2)
         lqm=0.d0     ! ln(-q^2/mu^2) choosen MSb scale mu^2=Q^2=-q^2
         lox=log(xi)  ! ln(-q^2/mq^2)
         f1ser=0.33333333333333331d0*((3360.d0*lox-4264.d0)*z6
     1   +(960.d0*lox-1184.d0)*z5+(288.d0*lox-336.d0)*z4
     2   +(96.d0*lox-96.d0)*z3+48.d0*lox*z2+24.d0*z+4.d0)*fac
      else if (1/y.lt.1.d0) then
         z=-1.d0/y                  ! z=q**2/(4mp(i)**2)
         z1=z
         z2=z*z
         z3=z2*z
         z4=z3*z
         z5=z4*z
         z6=z5*z
         z7=z6*z
         f1ser  = -1.0d0*z*(
     &    0.56754638550304182d0*z7+0.59907674025321089d0*z6
     1   +0.63651903651903652d0*z5+0.68198468198468198d0*z4
     2   +0.73881673881673882d0*z3+0.81269841269841270d0*z2
     3   +0.91428571428571426d0*z +1.0666666666666667d0)*fac
       endif
      if (y.le.eps) then            ! high Q^2 (light quark)
         f1=f1ser
      else if (1/y.le.eps) then     ! low Q^2 (heavy quark)
         f1=f1ser
      else
         rt=sqrt(1.d0+y)
         xi=(rt+1.d0)/(rt-1.d0)
         f1=1.d0-1.5d0*y+0.75d0*y**2/rt*log(xi)
      endif
      return
      end

      function f2(x,eps,i)
c calculate two-loop coefficient H_2
c files to be linked: ddilog,dli3
      implicit none
      logical test
      INTEGER ini,i,nl
      real *8 pi,pi2,ln2,zeta2,zeta3,zeta4,zeta5,b4,i3
      real *8 CF,CA,TF,fac,cc1
      real *8 null,one,two,half,four,qua,three,
     & threehalf,sixth,third,twothird,five,six,seven,eight,
     & ipi,ipi2,twozeta3,fourzeta3,threehalfzeta2,thirteensixth,
     & twozeta2,twoln2,halfzeta3,fourthird,eightthird
      real *8 f2,x,y,z,z1,z2,z3,z4,z5,z6,z7
      real *8 xin,lnx,lnx2,lox,lqm,eps
      real *8 rt,rtomy,xi,xi2,dpo3,dpo2,dli3,ddilog,clausen3,clausen2,
     & fx,fx2,gpzh,xx,yy,caphi,tcp,fcp,omx,y2,y3
      REAL *8 MQ(6),MP(6),TH(6)
      real *8 f1ser,f2ser,f2pad,f3ser,f3pad,rtz,omega,eps1,crad
      common /fser/f1ser,f2ser,f2pad,f3ser,f3pad,eps1,crad
C Input: Quark masses MQ and physical thresholds TH for switching NF
      COMMON /QUARK/MQ,MP,TH
      common /numer/pi,pi2,ln2,zeta2,zeta3,zeta4,zeta5,b4,i3
      common /group/CF,CA,TF,nL
      common /flag/test
      common /numbers/null,one,two,half,four,qua,three,
     & threehalf,sixth,third,twothird,five,six,seven,eight,
     & ipi,ipi2,twozeta3,fourzeta3,threehalfzeta2,thirteensixth,
     & twozeta2,twoln2,halfzeta3,fourthird,eightthird,ini
      save /numbers/
      data null,one,two,half,four,qua/0.d0,1.d0,2.d0,.5d0,4.d0,.25d0/
      data three,threehalf,sixth/3.d0,1.5d0,.1666666666666667d0/
      data third,twothird/.3333333333333333d0,.6666666666666667d0/
      data five,six,seven,eight/5.d0,6.d0,7.d0,8.d0/
      data ini/0/
      if (ini.eq.0) goto 2
C     crad=.2d0  ! convergence radius for Pade
C     crad=one
C     x=mp(i)**2/Q**2
 1    y=4.d0*x
      fac=3.d0/4.d0
      if (y.le.1.d0) then           ! high Q^2 (light quark)
         xin=1.d0/x
         z=-y                       ! z=4mp(i)**2/q**2
         z1=z
         z2=z1*z
         z3=z2*z
         z4=z3*z
         z5=z4*z
         z6=z5*z
         lnx=log(x)
         lnx2=lnx*lnx
         f2ser=
     &    (-0.99739583333333337d0*lnx2-2.1715060763888889d0*lnx
     1   -0.56754701967592591d0)*z6+(-1.0147569444444444d0*lnx2
     2   -2.1911747685185183d0*lnx-0.3246484375d0)*z5
     3   +(-1.0572916666666667d0*lnx2-2.2352430555555554d0*lnx
     4   +0.15719039351851852d0)*z4+(-1.2083333333333333d0*lnx2
     5   -2.1111111111111112d0*lnx+1.287037037037037d0)*z3
     6   +(-1.5d0*lnx2-0.25d0*lnx+2.0d0*zeta3
     7   +0.70833333333333337d0)*z2+3.0d0*lnx*z+3.0d0*z+1.0d0
         f2ser=f2ser*cf*fac
      else if (y.ge.1.d0) then
         z=-1.d0/y                 ! z=q**2/(4mp(i)**2)
         z1=z
         z2=z*z
         z3=z2*z
         z4=z3*z
         z5=z4*z
         z6=z5*z
         z7=z6*z
         f2ser  = -1.0d0*z*(
     &    7.4846105184863605d0*z7+7.3197660616542972d0*z6
     1   +7.1169487350461935d0*z5+6.8596989280064937d0*z4
     2   +6.5198980991573583d0*z3+6.0448314436885866d0*z2
     3   +5.3214814814814817d0*z +4.0493827160493829d0)
         f2ser=f2ser*cf*fac
       endif
* low q^2 - > omega -> Pade
C    omega=(1-Sqrt[1-z])/(1+Sqrt[1-z)])
C    where z=s/(4m^2)
      if (y.le.crad) then           ! high Q^2 (light quark)
         f2pad=f2ser
      else if (y.ge.crad) then
         z    =-1.d0/y              ! z=(4mp(i)**2)/q^2
         rtz  =sqrt(1.d0-z)
         omega=(1.d0-rtz)/(1.d0+rtz)
         f2pad=(
     &        (-23.38268397625945280752575281d0*
     &        (0.79512795906782063033208095d0 + omega)*
     &        (0.94179380254255339527605317d0 +
     &        1.72760955278441806235339204d0*omega + omega**2))/
     &        ((-3.77011740155411588432508642321923d0 + omega)*
     &        (-1.28771040999158820427823603613309d0 + omega)*
     &        (-0.89068739732974305251681975534801d0 + omega)))
         f2pad=f2pad/y*cf*fac
       endif
      if (y.le.eps) then            ! high Q^2 (light quark)
         f2=f2ser
      else if (1/y.le.eps) then     ! low Q^2 (heavy quark)
         f2=f2ser
      else
      Y=-Y
      OMX=ONE-ONE/Y
      IF (Y.LT.ONE) THEN
            RTOMY=DSQRT(ONE-Y)
            XI=(RTOMY-ONE)/(RTOMY+ONE)
            XI2=XI**2
            DPO3 = TWO*DLI3(XI)-DLI3(XI2)
            DPO2 = DDILOG(XI)-DDILOG(XI2)
            FX   = -HALF*DLOG(DABS(XI))
            FX2  = FX**2
            GPZH = DLOG(ONE-XI)+TWO*DLOG(ONE+XI)
            XX=DPO3+EIGHTTHIRD*FX*DPO2+FOURTHIRD*FX2*GPZH
            YY=EIGHTTHIRD*(DPO2+TWO*FX*GPZH+THREE*FX2)
      ELSE
            CAPHI=DASIN(DSQRT(one/Y))
            TCP=TWO*CAPHI
            FCP=FOUR*CAPHI
            DPO3 = TWO*CLAUSEN3(TCP)-CLAUSEN3(FCP)
            DPO2 = CLAUSEN2(TCP)-CLAUSEN2(FCP)
            FX   = -CAPHI
            FX2  = -CAPHI**2
            GPZH = DLOG(TWO*DSQRT(one/Y))+TWO*DLOG(TWO*DSQRT(OMX))
            XX=DPO3-EIGHTTHIRD*FX*DPO2+FOURTHIRD*FX2*GPZH
            YY=EIGHTTHIRD*(DPO2+TWO*FX*GPZH)
      ENDIF
      Y2=Y*Y
      Y3=Y*Y2
      RT=DSIGN(ONE,ONE-Y)*DSQRT(DABS(ONE-Y))
      f2=(3.d0*RT*(Y2-one)*YY+FX2*(9.d0*Y3-9.d0*Y2-12.d0*Y+24.d0)
     1   +6.d0*ZETA3*Y3+6.d0*XX*(Y2-Y3)+(11.d0-6.d0*ZETA3)*Y2
     2   +14.d0*FX*RT*(Y-Y2)-8.d0*Y-3.d0)/(Y-one)/3.d0
      endif
      return
 2    IPI =ONE/PI
      IPI2=IPI**2
      TWOZETA3=TWO*ZETA3
      FOURZETA3=FOUR*ZETA3
      THREEHALFZETA2=THREEHALF*ZETA2
      THIRTEENSIXTH=13D0*SIXTH
      TWOZETA2=TWO*ZETA2
      TWOLN2=TWO*LN2
      HALFZETA3=HALF*ZETA3
      FOURTHIRD=FOUR*THIRD
      EIGHTTHIRD=EIGHT*THIRD
      ini=1
      goto 1
      end

      subroutine dthree(x,i,dpiv0,dpiv1,dpiv2)
C vector current correlator up to three-loops in large Q^2>>mq^2 expansion
C up to (Q^2/mq^2)^6: Chetyrkin et al. NPB503 (1997) 339, Sec.3
C -sd/ds Pi^{(i)}*(12 pi^2) -> Adler function
C For e.m. current: take sum Q_f^2 N_{cf}
      implicit none
      integer i,nl,nf
      real *8 pi,pi2,ln2,zeta2,zeta3,zeta4,zeta5,b4,i3
      real *8 CF,CA,TF,fac,cc1,cc2,cc12
      real *8 dpiv0,dpiv1,dpiv2,dpiv2a,dpiv2n,dpiv2l,dpiv2f,x,xi,lqm,z
      real *8 lox,z1,z2,z3,z4,z5,z6
      real *8 dpiv1os1,dpiv2os1,dpiv2os2
      real *8 ALINP,EINP,als,adp,rmstoos,rostoms,zos,los,los2,lox2
      REAL *8 MQ(6),MP(6),TH(6)
      real *8 f1ser,f2ser,f2pad,f3ser,f3pad,eps,crad
      common /fser/f1ser,f2ser,f2pad,f3ser,f3pad,eps,crad
C Input: Quark masses MQ and physical thresholds TH for switching NF
      COMMON /QUARK/MQ,MP,TH
      COMMON /PARAM/ALINP,EINP,als,adp
      common /numer/pi,pi2,ln2,zeta2,zeta3,zeta4,zeta5,b4,i3
      common /group/CF,CA,TF,nL
      if (abs(x).lt.0.25d0) then
      nf=i
      nl=i-1
C     x=mp(i)**2/Q**2
* quantities in on-shell mass
      xi=1.d0/x
      los=log(xi)  ! ln(-q^2/mq^2)
      los2=los*los
      zos=-x       ! z=mp(i)**2/q**2
* need msbarmass in terms of os-mass:
C      call mstoosmass(xi,nf,adp,rmstoos)
C      x=x*rmstoos**2
C      call ostomsmass(xi,nf,adp,rostoms)
C      x=x/rostoms**2
      xi=1.d0/x
      z=-x      ! z=mp(i)**2/q**2
C      z=0.d0
C      zos=0.d0
      z1=z
      z2=z*z
C      z2=0.d0
      z3=z2*z
      z4=z3*z
      z5=z4*z
      z6=z5*z
C assume Q^2=-q^2=mu^2 together with alpha_s(Q^2)
      lqm=0.d0     ! ln(-q^2/mu^2) choosen MSb scale mu^2=Q^2=-q^2
      lox=log(xi)  ! ln(-q^2/mq^2)
      lox2=lox*lox
      cc1=CF*(4.d0+3.d0*(LOX-LQM))
      cc2=CF*CA*(1111.d0/24.d0-8.d0*zeta2-4.d0*I3+185.d0/6.d0*(LOX-LQM)
     &   +11.d0/2.d0*(LOX-LQM)**2)-CF*TF*nf*(71.d0/6.d0+8.d0*zeta2
     &   +26.d0/3.d0*(LOX-LQM)+2.d0*(LOX-LQM)**2)+CF**2*(121.d0/8.d0
     &   +30.d0*zeta2+8.d0*I3+27.d0/2.d0*(LOX-LQM)
     &   +9.d0/2.d0*(LOX-LQM)**2)-12.d0*CF*TF*(1.d0-2.d0*zeta2)
       cc12=cc1*cc1
C      cc1=0.d0
C      cc2=0.d0
C begin test
C for numerical derivative we need q^2--dependence of lqm
C      lqm=lox
C end test
C normalization for Adler function of e.m. current: sum Q_f^2 N_{cf} dpivx
C = sum Q_f^2 N_{cf} (12 pi^2) 1/3 Pi-bar of Chetyrkin et al.
C -> (12 pi**2)/(16 pi**2)=3/4
      fac=3.d0/4.d0
      dpiv0 =
     &    (3360.d0*LOX-4264.d0)*z6/3.d0+(960.d0*LOX-1184.d0)*z5/3.d0
     1   +(96.d0*LOX-112.d0)*z4+(32.d0*LOX-32.d0)*z3+16.d0*LOX*z2
     2   +8.d0*z+4.0d0/3.0d0
      dpiv0=dpiv0*fac
C
      dpiv1 =
     &    z2*(96.d0*ZETA3+72.d0*LOX**2+(132.d0-144.d0*LQM)*LOX
     1   +72.d0*LQM-62.d0)/3.d0+(4046400.d0*LOX**2
     1   +(5307180.d0-6804000.d0*LQM)*LOX
     2   +9768600.d0*LQM-14593954.d0)*z6/675.d0+(918600.d0*LOX**2
     3   +(1352540.d0-1620000.d0*LQM)*LOX+2322000.d0*LQM-3320397.d0)
     4   *z5/675.d0+(16488.d0*LOX**2+(28308.d0-31104.d0*LQM)*LOX
     5   +44064.d0*LQM-56579.d0)*z4/54.d0+(1800.d0*LOX**2
     6   +(3648.d0-3888.d0*LQM)*LOX+5184.d0*LQM-4688.d0)*z3/27.d0
     7   +(28.d0-12.d0*LQM)*z+1.d0
C MSb -> OS mass contribution from renormalization of 1-loop
      dpiv1os1 =-0.33333333333333331d0*cc1*(
     &    (10080.0d0*LOX-14472.0d0)*Z6+(2400.0d0*LOX-3440.0d0)*Z5
     1   +(576.0d0*LOX-816.0d0)*Z4+(144.0d0*LOX-192.0d0)*Z3
     2   +(48.0d0*LOX-24.0d0)*Z2+12.0d0*Z)
      dpiv1=(dpiv1*Cf+dpiv1os1)*fac
C
      dpiv2a =
     &    z5*(622080000000.d0*ZETA5-223948800000.d0*ZETA4
     1   +(553417920000.d0*LOX-795528216000.d0)*ZETA3+423438768000.d0
     2   *LOX**3+(963035751600.d0-1190505600000.d0*LQM)*LOX**2
     3   +(1049760000000.d0*LQM**2-1311681600000.d0*LQM+96433370460.d0)
     4   *LOX-1714608000000.d0*LQM**2+4703968080000.d0*LQM
     4   +37324800000.d0
     5   *B4-3504665218369.d0)/1.1664d8+z6*(233280000000.d0*ZETA5
     6   -65318400000.d0*ZETA4+(196445088000.d0*LOX-314820194400.d0)
     6   *ZETA3+185232506400.d0*LOX**3+(432801692100.d0-524413440000.d0
     7   *LQM)*LOX**2+(440899200000.d0*LQM**2-525253248000.d0*LQM
     8   -45837389640.d0)*LOX-706488480000.d0*LQM**2
     9   +2023595006400.d0*LQM+10886400000.d0*B4-1549887396739.d0)
     :   /9720000.d0+z4*(4561920.d0*ZETA5-2239488.d0*ZETA4+(4281984.d0
     ;   *LOX-5186880.d0)*ZETA3+2577648.d0*LOX**3+(5776176.d0-7122816.d0
     <   *LQM)*LOX**2+(6718464.d0*LQM**2-8947584.d0*LQM+3179838.d0)*LOX
     =   -11197440.d0*LQM**2+27895968.d0*LQM+373248.d0*B4-18833071.d0)
     >   /3888.d0+z3*(311040.d0*ZETA5-186624.d0*ZETA4+(241056.d0*LOX
     ?   -118260.d0)*ZETA3+104256.d0*LOX**3+(225252.d0-291600.d0*LQM)
     @   *LOX**2+(314928.d0*LQM**2-414072.d0*LQM+337536.d0)*LOX
     1   -524880.d0*LQM**2+979776.d0*LQM+31104.d0*B4-486019.d0)/972.d0
     2   -z*(560.d0*ZETA5+40.d0*ZETA3-216.d0*LQM**2+1044.d0*LQM-2279.d0)
     3   /24.d0+z2*(-40.d0*ZETA5-96.d0*ZETA4+(48.d0*LOX-96.d0*LQM
     4   +216.d0)*ZETA3+24.d0*LOX**3+(33.d0-72.d0*LQM)*LOX**2
     5   +(72.d0*LQM**2-66.d0
     5   *LQM+72.d0)*LOX-72.d0*LQM**2+131.d0*LQM+16.d0*B4+60.d0)
     5   +(-1.0d0)/8.0d0
      dpiv2a=dpiv2a*Cf**2
C
      dpiv2n =
     &    -z6*(116640000000.d0*ZETA5-65318400000.d0*ZETA4
     1   +(231880320000.d0*LOX-272683843200.d0)*ZETA3-19822564800.d0
     1   *LOX**3+(106824960000.d0*LQM-278498881800.d0)*LOX**2
     2   +(-89812800000.d0*LQM**2+404104752000.d0*LQM-576945947772.d0)
     3   *LOX+128945520000.d0*LQM**2-764302065600.d0*LQM
     4   +10886400000.d0*B4+1063309991687.d0)/1.944d7
     5   -z5*(12441600000.d0*ZETA5-8957952000.d0*ZETA4+(26715225600.d0
     6   *LOX-31431291840.d0)*ZETA3-2025072000.d0*LOX**3+(11640499200.d0
     7   *LQM-28979499600.d0)*LOX**2+(-10264320000.d0*LQM**2
     8   +47310266880.d0*LQM-69472708860.d0)*LOX+14712192000.d0*LQM**2
     9   -85320998784.d0*LQM+1492992000.d0*B4+112092841523.d0)
     9   /9331200.d0
     :   -z4*(2280960.d0*ZETA5-2239488.d0*ZETA4+(5308416.d0*LOX
     ;   -6902784.d0)*ZETA3-373632.d0*LOX**3+(2176416.d0*LQM-4986168.d0)
     <   *LOX**2+(-2052864.d0*LQM**2+9770832.d0*LQM-13950756.d0)*LOX
     =   +2908224.d0*LQM**2-16016844.d0*LQM+373248.d0*B4+19930795.d0)
     >   /7776.d0-z3*(103680.d0*ZETA5-124416.d0*ZETA4+(204768.d0*LOX
     ?   -357912.d0)*ZETA3-16896.d0*LOX**3+(79200.d0*LQM-170424.d0)
     @   *LOX**2+(-85536.d0*LQM**2+411936.d0*LQM-523878.d0)*LOX
     1   +114048.d0*LQM**2-541504.d0*LQM+20736.d0*B4+707407.d0)
     1   /1296.d0-z2*(3240.d0
     2   *ZETA5-2592.d0*ZETA4+(1296.d0*LOX+1584.d0*LQM-7788.d0)*ZETA3
     3   -396.d0*LOX**3+(1188.d0*LQM-2736.d0)*LOX**2+(-1188.d0*LQM**2
     4   +5670.d0*LQM-6468.d0)*LOX+594.d0*LQM**2-2769.d0*LQM+432.d0*B4
     5   +5677.d0)/54.d0-Z*(680.d0*ZETA5-128.d0*ZETA3-132.d0*LQM**2
     6   +1004.d0*LQM-2187.d0)/24.d0-(88.d0*ZETA3+22.d0*LQM-123.d0)
     6   /24.d0
      dpiv2n=dpiv2n*Cf*Ca
C
      dpiv2l =
     &    z2*((288.d0*LQM-1488.d0)*ZETA3-72.d0*LOX**3+(216.d0*LQM
     1   -360.d0)*LOX**2+(-216.d0*LQM**2+756.d0*LQM-888.d0)*LOX+108.d0
     2   *LQM**2-366.d0*LQM+346.d0)/27.d0+z6*(32832000.d0*ZETA3
     3   -20988000.d0*LOX**3+(121392000.d0*LQM-241267500.d0)*LOX**2
     4   +(-102060000.d0*LQM**2+329315400.d0*LQM-454799070.d0)*LOX
     5   +146529000.d0*LQM**2-682033620.d0*LQM+838963397.d0)/60750.d0
     6   +z5*(9072000.d0*ZETA3-14598000.d0*LOX**3+(82674000.d0*LQM
     7   -161200800.d0)*LOX**2+(-72900000.d0*LQM**2+243228600.d0*LQM
     8   -357290670.d0)*LOX+104490000.d0*LQM**2-472985730.d0*LQM
     9   +542452391.d0)/182250.d0-z4*(20736.d0*ZETA3+12192.d0*LOX**3
     9   +(124536.d0-65952.d0*LQM)*LOX**2+(62208.d0*LQM**2-216912.d0
     :   *LQM+337228.d0)*LOX-88128.d0*LQM**2+373196.d0*LQM-370773.d0)
     ;   /648.d0-z3*(3456.d0*ZETA3+384.d0*LOX**3+(3312.d0-1800.d0*LQM)
     <   *LOX**2+(1944.d0*LQM**2-6888.d0*LQM+11382.d0)*LOX
     =   -2592.d0*LQM**2+9008.d0*LQM-5974.d0)/81.d0+(8.d0*ZETA3+2.d0
     >   *LQM-11.d0)/6.d0-(12.d0*LQM**2-76.d0*LQM+147.d0)*z/6.d0
      dpiv2l=dpiv2l*Cf*Tf*nl
C
      dpiv2f =
     &    z2*((432.d0*LOX+288.d0*LQM+1752.d0)*ZETA3-72.d0*LOX**3+(
     1   216.d0*LQM-414.d0)*LOX**2+(-216.d0*LQM**2+756.d0*LQM-2598.d0
     2   )*LOX+108.d0*LQM**2-366.d0*LQM-194.d0)/27.d0+z6*(370908000.d0
     3   *ZETA3-163278000.d0*LOX**3+(80928000.d0*LQM-87094500.d0)*LOX
     4   **2+(-68040000.d0*LQM**2+219543600.d0*LQM+52967230.d0)*LOX
     5   +97686000.d0*LQM**2-454689080.d0*LQM+1807914031.d0)/40500.d0
     6   +z5*(283716000.d0*ZETA3-45405000.d0*LOX**3+(41337000.d0*LQM
     7   -126469800.d0)*LOX**2+(-36450000.d0*LQM**2+121614300.d0*LQM
     8   -202190805.d0)*LOX+52245000.d0*LQM**2-236492865.d0*LQM
     9   +929621779.d0)/91125.d0+z4*(482112.d0*ZETA3-35520.d0*LOX**3
     :   +(65952.d0*LQM-193872.d0)*LOX**2+(-62208.d0*LQM**2+216912.d0
     ;   *LQM-654580.d0)*LOX+88128.d0*LQM**2-373196.d0*LQM+1300905.d0)
     ;   /648.d0
     <   +z3*(30240.d0*ZETA3-1008.d0*LOX**3+(5400.d0*LQM-4716.d0)*LOX
     =   **2+(-5832.d0*LQM**2+20664.d0*LQM-58626.d0)*LOX+7776.d0*LQM
     >   **2-27024.d0*LQM+92972.d0)/243.d0+z*(96.d0*ZETA3-12.d0*LQM**2
     ?   +76.d0*LQM-275.d0)/6.d0+(8.d0*ZETA3+2.d0*LQM-11.d0)/6.d0
C MSb -> OS mass contribution from renormalization of 1-loop
      dpiv2os1 =
     &  -0.16666666666666666d0*((CC2*(5040.0d0*LOX-7236.0d0)+CC1**2*(5
     1   2074.0d0-32760.0d0*LOX))*Z6+(CC2*(1200.0d0*LOX-1720.0d0)+CC1*
     2   *2*(10660.0d0-6600.0d0*LOX))*Z5+(CC2*(288.0d0*LOX-408.0d0)+CC
     3   1**2*(2124.0d0-1296.0d0*LOX))*Z4+(CC2*(72.0d0*LOX-96.0d0)+CC1
     4   **2*(408.0d0-252.0d0*LOX))*Z3+(CC2*(24.0d0*LOX-12.0d0)+CC1**2
     5   *(54.0d0-60.0d0*LOX))*Z2+(6.0d0*CC2-9.0d0*CC1**2)*Z)
C MSb -> OS mass contribution from renormalization of 2-loop
C      dpiv2os2 =4.d0/3.d0*7.407407407407407d-4*CC1*(
C     &    (Z2*(-43200.0d0*ZETA3+(64800.0d0*LOX-64800.0d0)*LQM
C     1   -32400.0d0*LOX**2-27000.0d0*LOX+57600.0d0)
C     2   +((4.0824d7*LOX-6.54156d7)*LQM-2.42784d7*LOX**2-2.375028d7*LOX
C     3   +9.2870904d7)*Z6+((8100000.0d0*LOX-1.323d7)*LQM-4593000.0d0
C     4   *LOX**2-4925500.0d0*LOX+1.7954525d7)*Z5+((1555200.0d0*LOX
C     5   -2592000.0d0)*LQM-824400.0d0*LOX**2-1003200.0d0*LOX+3182800.0d0
C     6   )*Z4+((291600.0d0*LOX-486000.0d0)*LQM-135000.0d0*LOX**2
C     7   -183600.0d0*LOX+442800.0d0)*Z3+(8100.0d0*LQM-18900.0d0)*Z))
      dpiv2os2 =
     &    4.9382716049382717d-4*(CC1*Z2*(-86400.0d0*ZETA3+(129600.0d0*
     1   LOX-129600.0d0)*LQM-64800.0d0*LOX2-54000.0d0*LOX+115200.0d0)+
     2   CC1*((8.1648d7*LOX-1.308312d8)*LQM-4.85568d7*LOX2-4.750056d7*
     3   LOX+1.85741808d8)*Z6+CC1*((1.62d7*LOX-2.646d7)*LQM-9186000.0d
     4   0*LOX2-9851000.0d0*LOX+3.590905d7)*Z5+CC1*((3110400.0d0*LOX
     5   -5184000.0d0)*LQM-1648800.0d0*LOX2-2006400.0d0*LOX+6365600.0d
     6   0)*Z4+CC1*((583200.0d0*LOX-972000.0d0)*LQM-270000.0d0*LOX2-
     7   367200.0d0*LOX+885600.0d0)*Z3+CC1*(16200.0d0*LQM-37800.0d0)*Z
     8   )
       dpiv2f=dpiv2f*Cf*Tf
C       dpiv2os1=0.d0
C       dpiv2os2=
C     &    -0.33333333333333331d0*Z2*(96.0d0*CC1*CF*ZETA3+(144.0d0*CC1*
C     1   CF-144.0d0*CC1*CF*LOX)*LQM+72.0d0*CC1*CF*LOX2+(60.0d0*CC1*CF+
C     2   12.0d0*CC2-30.0d0*CC12)*LOX-128.0d0*CC1*CF-6.0d0*CC2+27.0d0*C
C     3   C12)+0.0044444444444444444d0*((6804000.0d0*CC1*CF*LOX-1.09026
C     4   d7*CC1*CF)*LQM-4046400.0d0*CC1*CF*LOX2+(-3958380.0d0*CC1*CF-1
C     5   89000.0d0*CC2+1228500.0d0*CC12)*LOX+1.5478484d7*CC1*CF+271350
C     6   .0d0*CC2-1952775.0d0*CC12)*Z6+0.018518518518518517d0*((3240
C     7   00.0d0*CC1*CF*LOX-529200.0d0*CC1*CF)*LQM-183720.0d0*CC1*CF*LOX*
C     8   *2+(-197020.0d0*CC1*CF-10800.0d0*CC2+59400.0d0*CC12)*LOX+7181
C     9   81.0d0*CC1*CF+15480.0d0*CC2-95940.0d0*CC12)*Z5+0.0370370370
C     :   37037035d0*((31104.0d0*CC1*CF*LOX-51840.0d0*CC1*CF)*LQM-16488.0
C     ;   d0*CC1*CF*LOX2+(-20064.0d0*CC1*CF-1296.0d0*CC2+5832.0d0*CC1
C     <   2)*LOX+63656.0d0*CC1*CF+1836.0d0*CC2-9558.0d0*CC12)*Z4+((21
C     =   6.0d0*CC1*CF*LOX-360.0d0*CC1*CF)*LQM-100.0d0*CC1*CF*LOX2+(-13
C     >   6.0d0*CC1*CF-12.0d0*CC2+42.0d0*CC12)*LOX+328.0d0*CC1*CF+16.0d
C     ?   0*CC2-68.0d0*CC12)*Z3+0.5d0*(12.0d0*CC1*CF*LQM-28.0d0*CC1*C
C     @   F-2.0d0*CC2+3.0d0*CC12)*Z
       dpiv2=(dpiv2a+dpiv2n+dpiv2l+dpiv2f+dpiv2os1+dpiv2os2)*fac
* on-shell high Q^2 expansion (2 terms) for test
C       f3ser= (((16.d0*NL-248.d0)*ZETA3+(4.d0*LQM-22.d0)*NL
C     1   -62.d0*LQM+343.d0)/18.d0-Zos*(4180.d0*ZETA5-856.d0*ZETA3
C     1   +(-144.d0*NL+288.d0*LN2+864.d0)*ZETA2+((72.d0*LOS-72.d0)*LQM
C     2   -36.d0*LOS2-156.d0*LOS+228.d0)*NL+1116.d0*(1.d0-LOS)*LQM
C     3   +126.d0*LOS2+4110.d0*LOS-6742.d0)/27.d0)*fac
      else
       dpiv0=0.d0
       dpiv1=0.d0
       dpiv2=0.d0
       f3pad=0.d0
      endif
      return
      end

      subroutine dthreelow(x,i,dPiv00,dPiv10,dPiv20)
      implicit none
      integer i,nl
      real *8 pi,pi2,ln2,zeta2,zeta3,zeta4,zeta5,b4,i3
      real *8 Cf,Ca,Tf,fac
      real *8 x,z,dPiv00,dPiv10,dPiv20,lmm,xi
      real *8 Cv21,Cv22,Cv23,Cv24,Cv25,Cv26,Cv27,Cv28
      real *8 z1,z2,z3,z4,z5,z6,z7,z8
      real *8 f1ser,f2ser,f2pad,f3ser,f3pad,eps,crad
      common /fser/f1ser,f2ser,f2pad,f3ser,f3pad,eps,crad
      common /numer/pi,pi2,ln2,zeta2,zeta3,zeta4,zeta5,b4,i3
      common /group/CF,CA,TF,nL
      if (x.gt.0.25d0) then
C     x=mp(i)**2/Q**2
      xi=1.d0/x
      z=-xi/4.d0      ! z=q**2/(4mp(i)**2)
      z1=z
      z2=z1*z
      z3=z2*z
      z4=z3*z
      z5=z4*z
      z6=z5*z
      z7=z6*z
      z8=z7*z
C assume Q^2=-q^2=mu^2 together with alpha_s(Q^2)
C     lmm := ln(mu^2/mq^2) choosen MSb scale mu^2=Q^2=-q^2
      lmm=log(xi)
      nl=i-1
C normalization for e.m. current: sum Q_f^2 N_{cf} ppivx
C = sum Q_f^2 N_{cf}  1/3 Pi-bar of Chetyrkin et al.
C -> 12 pi**2 * 1/(16 pi**2)
      fac=3.d0/4.d0
      dPiv00
     &  = -1.0d0*Z*(
     &    0.56754638550304182d0*Z7+0.59907674025321089d0*Z6
     1   +0.63651903651903652d0*Z5+0.68198468198468198d0*Z4
     2   +0.73881673881673882d0*Z3+0.81269841269841270d0*Z2
     3   +0.91428571428571426d0*Z +1.0666666666666667d0)*fac
      dPiv10
     &  = -1.0d0*Z*(
     &    7.4846105184863605d0*Z7+7.3197660616542972d0*Z6
     1   +7.1169487350461935d0*Z5+6.8596989280064937d0*Z4
     2   +6.5198980991573583d0*Z3+6.0448314436885866d0*Z2
     3   +5.3214814814814817d0*Z +4.0493827160493829d0)*fac*cf
      Cv21
     &  = CF**2*(13.183449074074074d0*ZETA3
     &   -6.4000000000000004d0*LN2*ZETA2+4.0d0*ZETA2
     1   -10.054398148148149d0)+CA*CF*(3.7786458333333335d0*ZETA3
     2   +3.2000000000000002d0*LN2*ZETA2-1.0666666666666667d0*ZETA2
     3   +3.711934156378601d0*LMM+0.66145833333333337d0)+CF*TF*(
     4    0.93981481481481477d0*ZETA3+2.1333333333333333d0*ZETA2
     4   -1.3497942386831276d0*LMM-3.9118655692729765d0)
     5   +CF*NL*TF*(-1.0666666666666667d0*ZETA2
     6   -1.3497942386831276d0*LMM-0.58436213991769548d0)
      Cv22
     &  = CF**2*(105.41638454861111d0*ZETA3-5.4857142857142858d0*LN2*ZET
     1   A2+3.4285714285714284d0*ZETA2-119.70823098851166d0)+CA*CF*(13.0
     2   84400318287036d0*ZETA3+2.7428571428571429d0*LN2*ZETA2-0.9142857
     3   1428571426d0*ZETA2+2.4390123456790125d0*LMM-11.040525795181756d
     4   0)+CF*TF*(0.77056206597222221d0*ZETA3+1.8285714285714285d0*ZETA
     5   2-0.88691358024691358d0*LMM-3.6670259452160492d0)+CF*NL*TF*(-0.
     6   91428571428571426d0*ZETA2-0.88691358024691358d0*LMM-0.670008230
     7   45267488d0)
      Cv23
     &  = CF**2*(640.7146054028708d0*ZETA3-4.8761904761904766d0*LN2*ZETA
     1   2+3.0476190476190474d0*ZETA2-762.93978937682573d0)+CA*CF*(50.02
     2   9998885866192d0*ZETA3+2.4380952380952383d0*LN2*ZETA2-0.81269841
     3   26984127d0*ZETA2+1.8470318300159569d0*LMM-56.184259631375042d0)
     4   +CF*TF*(0.89373553240740744d0*ZETA3+1.6253968253968254d0*ZETA2-
     5   0.6716479381876207d0*LMM-3.5981021519204388d0)+CF*NL*TF*(-0.812
     6   6984126984127d0*ZETA2-0.6716479381876207d0*LMM-0.55882161033725
     7   664d0)
      Cv24
     &  = CF**2*(3650.8505855330209d0*ZETA3-4.4329004329004329d0*LN2*ZET
     1   A2+2.7705627705627704d0*ZETA2-4381.3486678569998d0)+CA*CF*(195.
     2   90013933225495d0*ZETA3+2.2164502164502164d0*LN2*ZETA2-0.7388167
     3   3881673882d0*ZETA2+1.4941433143902281d0*LMM-232.0958935309009d0
     4   )+CF*TF*(1.0693109300401475d0*ZETA3+1.4776334776334776d0*ZETA2-
     5   0.54332484159644656d0*LMM-3.6165148096629678d0)+CF*NL*TF*(-0.73
     6   881673881673882d0*ZETA2-0.54332484159644656d0*LMM-0.45558683086
     7   098334d0)
      Cv25
     &  = CF**2*(19710.500011948116d0*ZETA3-4.0919080919080919d0*LN2*ZET
     1   A2+2.5574425574425574d0*ZETA2-23686.109567332016d0)+CA*CF*(771.
     2   93359053525319d0*ZETA3+2.045954045954046d0*LN2*ZETA2-0.68198468
     3   198468198d0*ZETA2+1.2576114701345238d0*LMM-924.95413975099541d0
     4   )+CF*TF*(1.2610707600911459d0*ZETA3+1.363969363969364d0*ZETA2-0
     5   .45731326186709959d0*LMM-3.6872181849399492d0)+CF*NL*TF*(-0.681
     6   98468198468198d0*ZETA2-0.45731326186709959d0*LMM-0.372543162722
     7   94996d0)
      Cv26
     &  = CF**2*(102105.66660275949d0*ZETA3-3.8191142191142191d0*LN2*ZET
     1   A2+2.3869463869463869d0*ZETA2-122729.96721804331d0)+CA*CF*(3050
     2   .3608848925337d0*ZETA3+1.9095571095571096d0*LN2*ZETA2-0.6365190
     3   3651903652d0*ZETA2+1.0873116122987239d0*LMM-3664.0916541401875d
     4   0)+CF*TF*(1.4577662506037288d0*ZETA3+1.273038073038073d0*ZETA2-
     5   0.39538604083589962d0*LMM-3.7907201039531002d0)+CF*NL*TF*(-0.63
     6   651903651903652d0*ZETA2-0.39538604083589962d0*LMM-0.30644968698
     7   54727d0)
      Cv27
     &  = CF**2*(512533.83177614817d0*ZETA3-3.5944604415192649d0*LN2*ZET
     1   A2+2.2465377759495406d0*ZETA2-616088.16093395511d0)+CA*CF*(1207
     2   7.364350535065d0*ZETA3+1.7972302207596325d0*LN2*ZETA2-0.5990767
     3   4025321089d0*ZETA2+0.95854079378806267d0*LMM-14515.334055781799
     4   d0)+CF*TF*(1.6549119018478158d0*ZETA3+1.1981534805064218d0*ZETA
     5   2-0.34856028865020461d0*LMM-3.9155955165879486d0)+CF*NL*TF*(-0.
     6   59907674025321089d0*ZETA2-0.34856028865020461d0*LMM-0.253267530
     7   5218518d0)
      Cv28
     &  = CF**2*(2510379.1031226483d0*ZETA3-3.4052783130182509d0*LN2*ZET
     1   A2+2.128298945636407d0*ZETA2-3017612.0408469159d0)+CA*CF*(47892
     2   .842170848846d0*ZETA3+1.7026391565091255d0*LN2*ZETA2-0.56754638
     3   550304182d0*ZETA2+0.85761162190989548d0*LMM-57567.797534720674d
     4   0)+CF*TF*(1.8505049583359829d0*ZETA3+1.1350927710060836d0*ZETA2
     5   -0.31185877160359837d0*LMM-4.0547961825454735d0)+CF*NL*TF*(-0.5
     6   6754638550304182d0*ZETA2-0.31185877160359837d0*LMM-0.2098378555
     7   4417581d0)

      dPiv20 =-(Cv21*z+2d0* Cv22*z2+3d0* Cv23*z3+4d0* Cv24*z4
     1  +5d0* Cv25*z5 +6d0* Cv26*z6+7d0* Cv27*z7 +8d0* Cv28*z8)*fac
      else
        dpiv00=0.d0
        dpiv10=0.d0
        dpiv20=0.d0
      endif
      return
      end

      subroutine dthreePad(x,i)
* input: x=mp(i)**2/Q**2,i flavor number; output f3pad via common "fser"
      implicit none
      integer i,nl
      real *8 dpiv2,dpiv2a,dpiv2n,dpiv2l,dpiv2f
      real *8 pi,pi2,ln2,zeta2,zeta3,zeta4,zeta5,b4,i3
      real *8 Cf,Ca,Tf,fac
      real *8 x,y,z,lmm,xi,rtz,omega,omega2,omega3
      real *8 f1ser,f2ser,f2pad,f3ser,f3pad,eps,crad
      common /fser/f1ser,f2ser,f2pad,f3ser,f3pad,eps,crad
      common /numer/pi,pi2,ln2,zeta2,zeta3,zeta4,zeta5,b4,i3
      common /group/CF,CA,TF,nL
C     crad=.2d0  ! convergence radius for Pade
C     x=mp(i)**2/Q**2
      y=4.d0*x
      xi=1.d0/x
C assume Q^2=-q^2=mu^2 together with alpha_s(Q^2)
C     lmm := ln(mu^2/mq^2) choosen MSb scale mu^2=Q^2=-q^2
      lmm=log(xi)
      nl=i-1
C normalization for e.m. current: sum Q_f^2 N_{cf} ppivx
C = sum Q_f^2 N_{cf}  1/3 Pi-bar of Chetyrkin et al.
C -> 12 pi**2 * 1/(16 pi**2)
      fac=3.d0/4.d0
* low q^2 - > omega -> Pade
C    omega=(1-Sqrt[1-z])/(1+Sqrt[1-z)])
C    where z=s/(4m^2)
      if (y.le.crad) then           ! high Q^2 (light quark)
         f3pad=f3ser
      else if (y.ge.crad) then
         z    =-1.d0/y              ! z=(4mp(i)**2)/q^2
         rtz  =sqrt(1.d0-z)
         omega =(1.d0-rtz)/(1.d0+rtz)
         omega2=omega*omega
         omega3=omega2*omega
C
      dpiv2a =
     & (5.075433058074201d0+34.13199318444022d0*omega
     & +68.02683012594772d0*omega2+45.657828406304d0*omega3)/
     & (1.d0-3.352347801958719d0*omega+4.080149215266563d0*omega2
     & -1.970259322298452d0*omega3)
      dpiv2a=dpiv2a*Cf**2
C
      dpiv2n =
     & (lmm*(3.711934156378601d0 + 11.47745319383203d0*omega
     & +12.5048784850192d0*omega2 + 4.95686865013471d0*omega3))/
     & (1.d0-2.164544206096246d0*omega+1.375657081049371d0*omega2
     & -0.2312605806230149d0*omega3)
     & +(7.097589817972592d0 + 32.89980102991667d0*omega
     & +47.03202731614738d0*omega2 + 22.35717117590982d0*omega3)/
     & (1.d0-2.4781458121294d0*omega+1.97798334431171d0*omega2
     & -0.5296210778397718d0*omega3)
      dpiv2n=dpiv2n*Cf*Ca
C
      dpiv2l =
     & (-2.338958477889137d0 - 11.4122654847818d0*omega
     & -16.76943138095181d0*omega2-8.137378105390544d0*omega3)/
     & (1.d0-2.556401988942401d0*omega+2.146658671128108d0*omega2
     & -0.6251186156509072d0*omega3) +
     & (lmm*(-1.349794238683127d0 - 4.17361934321165d0*omega
     & -4.547228540006983d0*omega2 - 1.802497690958076d0*omega3))/
     & (1.d0-2.164544206096246d0*omega+1.375657081049371d0*omega2
     & -0.2312605806230149d0*omega3)
      dpiv2l=dpiv2l*Cf*Tf*nl
C
      dpiv2f =
     & (lmm*(-1.349794238683127d0 - 4.17361934321165d0*omega
     & -4.547228540006983d0*omega2-1.802497690958076d0*omega3))/
     & (1.d0-2.164544206096246d0*omega+1.375657081049371d0*omega2
     & -0.2312605806230149d0*omega3) +
     & (0.7270379925097102d0 + 1.625728916129475d0*omega
     & +1.084040796475564d0*omega2 + 0.1809517106280248d0*omega3)/
     & (1.d0-0.7030920327505435d0*omega-0.4599548673448888d0*omega2
     & +0.167640770357784d0*omega3)
        dpiv2f=dpiv2f*Cf*Tf
        dpiv2=(dpiv2a+dpiv2n+dpiv2l+dpiv2f)*fac
        f3pad=dpiv2/y
      else
      endif
      return
      end

      subroutine ostomsmass(mdm,i,adpatmu,rostoms)
      implicit none
      integer i,nf,nl
      real *8 mdm,rostoms,adpatmu,Lmm,Lmm2,cc1,cc2,LOXMLQM
      real *8 pi,pi2,ln2,zeta2,zeta3,zeta4,zeta5,b4,i3
      real *8 CF,CA,TF
      common /numer/pi,pi2,ln2,zeta2,zeta3,zeta4,zeta5,b4,i3
      common /group/CF,CA,TF,nL
      nf=i
      LOXMLQM=log(mdm)
      cc1=CF*(4.d0+3.d0*(LOXMLQM))
      cc2=CF*CA*(1111.d0/24.d0-8.d0*zeta2-4.d0*I3+185.d0/6.d0*(LOXMLQM)
     &   +11.d0/2.d0*(LOXMLQM)**2)-CF*TF*nf*(71.d0/6.d0+8.d0*zeta2
     &   +26.d0/3.d0*(LOXMLQM)+2.d0*(LOXMLQM)**2)+CF**2*(121.d0/8.d0
     &   +30.d0*zeta2+8.d0*I3+27.d0/2.d0*(LOXMLQM)
     &   +9.d0/2.d0*(LOXMLQM)**2)-12.d0*CF*TF*(1.d0-2.d0*zeta2)
      rostoms=1.d0*adpatmu/4.d0*cc1*adpatmu**2/16.d0*cc2
      return
      end

      subroutine alphaQCD(s,alsMOM,alsMSb,ior)
* Input s=E^2, ior. Output: alphaMOM,alphaMSb
* Note ior < 5 implemented only
      implicit none
      character *1 cfn(4),cfm(4)
      character *6 dir
      INTEGER ini,i,j,k,ior,L,NCOM,NCURF,NSF
      PARAMETER (L=999)
      REAL *8 s,w,alsMOM,alsMSb,ANZCOM,ANZCUR,NUM
      REAL X(L,2),Y(L,2),Z(L,2)
      data cfn /'1','2','3','4'/
      data cfm /'1','2','2','2'/
      save
      dir='./sav/'
      data ini/0/
C FJ 02/06/08 ior=5 not implemented yet here 
      if (ior.gt.4) then
         write (*,*) ' Warning: alphaQCD requires ior < 5; we set it =4' 
         ior=4
      endif
      if (ini.eq.0) goto 2
 1    w=log(abs(s))  ! alpha-s is taken at space-like momentum transfer
      j=1
      if (w.le.x(j,1)) then
        write (*,*) ' warning: alpha-s: e < e_min '
        alsMOM=1.d0/y(j,1)
        alsMSb=1.d0/z(j,2)
      else if (w.ge.x(nsf,1)) then
        write (*,*) ' warning: alpha-s: e > e_max '
        alsMOM=1.d0/y(nsf,1)
        alsMSb=1.d0/z(nsf,2)
      else
         do while (x(j,1).le.w)
            j=j+1
         enddo
        alsMOM=Y(j-1,1)+(Y(j,1)-Y(j-1,1))/(X(j,1)-X(j-1,1))*(W-X(j-1,1))
        alsMOM=1.d0/alsMOM
        alsMSb=Z(j-1,2)+(Z(j,2)-Z(j-1,2))/(X(j,2)-X(j-1,2))*(W-X(j-1,2))
        alsMSb=1.d0/alsMSb
c        write (*,*) j,nsf,alsMOM,alsMSb
      endif
      return
 2    continue
C Files qcdpar4_alsi written with
  9   FORMAT(1X,F12.3)
 10   FORMAT (1X ,F10.2,2(2X,F9.4))
C     WRITE (1,10) log((e/lsc)**2),1.d0/ALFAmom,1.d0/ALFAMSb
C     .mo1,2,3,4 contain relevant BFMOM data in 2nd column
C     .ou1,2,3,4 contain relevant MSbar data in 3rd column
      OPEN(UNIT=19,FILE='./qcd/qcdpar4_alsi.mo'//cfm(ior),STATUS='OLD')
      OPEN(UNIT=20,FILE='./qcd/qcdpar4_alsi.ou'//cfn(ior),STATUS='OLD')
C
C Get data from DATA-FILES
C
      DO J=1,2
        K=18+J
        READ(K,9) ANZCOM
        READ(K,9) ANZCUR
        READ(K,9) NUM
        NCOM=NINT(ANZCOM)
        NCURF=NINT(ANZCUR)
        NSF=NINT(NUM)
        IF (NCOM.GT.2) THEN
           DO I=1,NCOM-2
              READ(K,*)
           ENDDO
        ENDIF
        DO I=1,NSF
C Read data
           READ(K,10,END=300,ERR=301) X(I,J),Y(I,J),Z(I,J)
c           WRITE (44,*) I,J,X(I,J),Y(I,J),Z(I,J)
        ENDDO
300     GOTO 302
301     PRINT *,' ERROR IN READING THE DATA '
302     CONTINUE
      close(k)
      ENDDO
      ini=1
      goto 1
      end

      subroutine getnfa(e,nf)
C find n_f effective for given energy scale e
C here adapted for Adler function: threshold at about 2.5*mpole 
      implicit none
      integer nf,i,j
      real *8 e
      real *8 mq(6),mp(6),th(6)
      common /quark/mq,mp,th
      nf=6
      do i=1,3
        if(e.lt.1.25d0*th(nf)) nf=nf-1
      enddo
      return
      end
c
c      subroutine analsEana(e,anals)
c      implicit none
c      real *8 pi,b0,nfx,e,Q2,fac1,fac2,c21,c22,Lambda21,Lambda22,
c     &     L1,L2,ellLc1_2,ellLc2_2,calA1_2,calA2_2,pi2
c      real *8 anals(3)
c      pi=4.d0*atan(1.d0)
c      pi2=pi*pi
c      nfx=3.d0
c      b0=11.d0-2.d0/3.d0*nfx
c      fac1=4.d0*pi/b0
cc      fac1=pi/b0
c      fac2=fac1*fac1
c      c21=-1.015d0
c      Lambda21=67.d-3            ! in MeV
c      c22=-1.544d0
c      Lambda22=34.5d-3           ! in MeV
c      Q2=e*e
c      if (e.gt.Lambda21) then 
c         L1=log(Q2/Lambda21**2)
c         ellLc1_2 = L1 + c21*log(sqrt((L1)**2+4.d0*pi2))
c         calA1_2=fac1*(1.d0/ellLc1_2+ 1.d0/(1.d0 - exp(ellLc1_2)))
c      else 
c         calA1_2=fac1
c      endif
c      if (e.gt.Lambda22) then
c         L2=log(Q2/Lambda22**2)
c         ellLc2_2 = L2 + c22*log(sqrt((L2)**2+4.d0*pi2))
c         calA2_2=fac2*(1.d0/ellLc2_2- 1.d0/(1.d0 - exp(ellLc2_2))**2
c     &        *exp(ellLc2_2))
c      else 
c         calA2_2=fac2
c      endif
c      anals(1)=calA1_2
c      anals(2)=calA2_2
c      return
c      end
c      
      subroutine analsEanax(e,Lam,anals)
c provides APT couplings of order 1,2 and 3      
      implicit none
      integer nfx
      real *8 pi,one,e,Q2,Lam,Lam2,L,L2,L3,logL,expL,exp2L,F1L,be0f,
     &     calA1_1,calA2_1,calA3_1,be1f,a,b
      real *8 be0(6),be1(6),anals(3) 
      common /rgcoe/be0,be1
      data one /1.d0/
      data pi /3.141592653589793d0/
      call getnfa(e,nfx)
      nfx=5
      be0f=be0(nfx)
      be1f=be1(nfx)
      Q2=e*e
      Lam2=Lam*Lam
      L=log(Q2/Lam2)
      b=be1f/be0f**2
      a=2.d0
      L=L+b*log(sqrt(L**2+a*pi**2))
      L2=L*L
      L3=L2*L
      logL=log(L)
      expL=exp(L)
      exp2L=exp(2.d0*L)
      F1L=one/(expL-one)
      calA1_1=(one/L-F1L)/be0f
      calA2_1=(one/L2-expL*F1L**2)/be0f**2
      calA3_1=(one/L3-0.5d0*(expL+exp2L)*F1L**3)/be0f**3
c      calA1_2= calA1_1+c1*(D1A1+c1*D2A1)      
c      calA2_2= calA2_1+2.d0*c1*(D1A2+c1*D2A2)
c      calA1L_2=one-c1*((logL-psi1)/L+psi1*calA1_1+)
c      calA2L_2=calA1_1-2.d0*c1*((logL-psi2)/L2+psi2*calA2_1+)
c      calA2L2_2=one+c1*(2.d0*D1A22+3.d0*c1*D2A22)
      anals(1)=calA1_1
      anals(2)=calA2_1
      anals(3)=calA3_1
      RETURN
      END
c      
      subroutine analsEdat(e,anals)
c provides APT couplings of order 1,2 and 3 using interpolating table
      implicit none
      integer n,i,ini,ij
      parameter(n=18)
      real *8 XX(n),YY(n,3)
      real *8 e,pi,beta0,beta1,nf,eps,dlog,anals(3)
C 18
        DATA XX/
     &     0.10D0,   0.20D0,   0.30D0,   0.40D0,   0.50D0,
     &     0.60D0,   0.70D0,   0.80D0,   0.90D0,   1.00D0,
     &     1.20D0,   1.40D0,   1.60D0,   1.80D0,   2.00D0,
     &     2.20D0,   2.40D0,   2.60D0/
        DATA YY/
     &  0.754824D0,0.635380D0,0.562731D0,0.511933D0,0.473770D0,
     &  0.443761D0,0.419392D0,0.399119D0,0.381931D0,0.367133D0,
     &  0.342862D0,0.323686D0,0.308076D0,0.295069D0,0.284028D0,
     &  0.274511D0,0.266205D0,0.258876D0,
     &  0.118112D0,0.124962D0,0.122311D0,0.117273D0,0.111773D0,
     &  0.106436D0,0.101465D0,0.096911D0,0.092763D0,0.088990D0,
     &  0.082421D0,0.076922D0,0.072264D0,0.068273D0,0.064815D0,
     &  0.061790D0,0.059121D0,0.056747D0,
     &  0.003872D0,0.000222D0,0.001702D0,0.003754D0,0.005540D0,
     &  0.006942D0,0.007999D0,0.008777D0,0.009349D0,0.006974D0,
     &  0.007820D0,0.008289D0,0.008539D0,0.008634D0,0.008651D0,
     &  0.008576D0,0.008487D0,-.015240D0/
c
        data ini /1/
        if (ini.eq.0) then
           pi=4.d0*datan(1.d0)
           nf=3.d0
           beta0=1.d0/(4.d0*pi)*(11.d0-2.d0/3.d0*nf)
           beta1=1.d0/(4.d0*pi)**2*(102.d0-38.d0/3.d0*nf)
           do i=1,n-1
              dlog= 2.d0*(log(XX(i+1))-log(XX(i)))
              YY(i,3)=-((YY(i+1,1)-YY(i,1))/dlog+beta0*YY(i,2))/beta1
              write (11,*) YY(i,3)
           enddo 
           i=n
           dlog= 2.d0*(log(3.d0)-log(2.d0))
           YY(i,3)=-((0.26444d0-0.30263d0)/dlog+
     &          beta0*0.07438d0)/beta1
           write (11,*) YY(i,3)
           write (*,*) ' Table entries for A_3 written to fort.11'
           ini=1
        endif
        ij=N
        eps=0.005d0
        IF ((E.GT.xx(1)).AND.(E.LT.xx(N))) THEN
           do while (xx(ij).ge.e)
              ij=ij-1
           enddo
           do i=1,3 
              anals(i)=YY(IJ,I)
     &             +(YY(IJ+1,I)-YY(IJ,I))/(XX(IJ+1)-XX(IJ))*(E-XX(IJ))
           enddo
        ELSE
           do i=1,3
              anals(i)=0.D0
              IF ((E.GT.(xx(1)-eps)).AND.(E.LT.(xx(1)+eps))) then
                 anals(i)=YY(1,I)
              ENDIF
              IF ((E.GT.(xx(N)-eps)).AND.(E.LT.(xx(N)+eps))) then
                 anals(i)=YY(N,I)
              ENDIF
           enddo
        ENDIF
        RETURN
        END
