C;;;;;;;;;;;;;;;;;;;;;;;;;;;;; -*- Mode: Fortran -*- ;;;;;;;;;;;;;;;;;;;;;;;;;;;
C;; qcdpar2.f -- 
C;; Copyright (C) F. Jegerlehner 1998. Email: <fjeger@ifh.de>
C;; 
C;; Author          : Fred Jegerlehner
C;; Created On      : Tue Aug 11 14:10:57 1998
C;; Last Modified By: Friedrich Jegerlehner
C;; Last Modified On: Mon Aug 20 00:27:28 2012
C;; Update Count    : 14
C;; Status          : Unknown, Use with caution!
C;; 
C;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
C*********************************************************************
C*                                                                   *
C*        SUBROUTINES FOR CALCULATION OF RUNNING PARAMETERS          *
C*                                                                   *
C*                  in QUANTUMCHROMODYNAMICS                         *
C*                                                                   *
C*            by F.Jegerlehner, Paul Scherrer Institute              *
C*                                                                   *
C*********************************************************************
C Subroutines: 
C           - ALQCD2(S,ALINP,EINP,MTOP,AL,NFX,FTQ,IOR)
C           - M2QCD2(S,ALINP,EINP,MTOP,M2,IQU,FTQ,IOR)
C
c Updates: 27/04/12 - adapted to alphaQED package inp,OUARK -> var1,quama
c
C INPUT PARAMETERS: S=E**2 (E=energy scale for which the QCD coupling
C (Alpha QCD) and the current quark masses are wanted; ALINP is alphas
C determined at scale EINP in the MS-bar scheme to order IOR in
C perturbation theory (experimental input); MTOP is the as yet unknown
C top-quark mass, which is defined such that the TOPONIUM-mass is given
C by M(toponium)=2*MTOP*(1+4/3*AL/PI+...). 
C terms.
C FTQ=0 mass fixed at fixed flavor threshold
C FTQ=1 running mass using  pole  mass 2*MQ*(1+4/3*al/pi+...) as threshold mass
C FTQ=2 running mass using MS-bar mass 2*MQ as threshold mass
C for s < 4 mass^2 take mass at threshold
C Use the function rpm to calculate the pole mass: 
C       MPOL=RR(I,I)*rpm(XTHR,I,AS(I),IOR)*MQ(I)
C IOR=0,1,2,3 order in perturbation theory
C
C OUTPUT PARAMETERS: 
C
C AL(Alhpa_QCD) from ALQCD2(S,ALINP,EINP,MTOP,AL,NF,FTQ,IOR) 
C
C is the running alpha at scale E=SQRT(S) in MS-bar scheme to order IOR
C in perturbation theory for NF(number of flavors with thresholds
C below E); 
C
C M2(quark mass squared) from M2QCD2(S,ALINP,EINP,MTOP,M2,IQU,FTQ,IOR) 
C
C where the quark flavor is indexed by IQU:1(u),2(d),3(s),4(c),5(b),6(t).
C Input values are choosen to be the current quark masses at 1GeV as given
C by J.Gasser & H.Leutwyler Phys.Rep.87C (1982) 77. They must be provided
C via the  COMMON /quama/MQ,MP,TH from the calling program.
C
       SUBROUTINE ALQCD2(S,ALINP,EINP,MTOP,AL,NFX,FTQ,IOR)
C      ###################################################
       IMPLICIT NONE
       INTEGER NFX,FTQ,IOR,IQ,NF,IOR1,j
       REAL *8 S,ALINP,EINP,MTOP,AL,SK,MTT,XTHR,TT3,TT4,TT5,TT6
     &        , ASQ,RMQ,LL,QQ,L,FIQ,RMS,RMS2
       REAL *8 EINP_old,ALIN_old,MTOP_old,FTQ_old,ALINP1,EINP1,MTOP1
       REAL *8 LFUN,RFUN
       REAL *8 PI,CS0,CS1,CS2,CM0,CM1,CM2
       REAL *8 MQ(6),MP(6),TH(6)
       REAL *8 AS(6),LA(6),RP(6),RM(6),RR(6,6),MS(6),TT(6)
       REAL *8 X0(6),X1(6),X2(6),Y0(6),Y1(6),Y2(6)
       COMMON /COEFF/CS0,CS1,CS2,CM0,CM1,CM2,IOR1
       COMMON /ASETC/ AS,LA,RP,RM,RR,MS,TT
C Input: Quark masses MQ and physical thresholds TH for switching NF
c provided by constants_qcd (via rqcdparam)
       common /var1/pi,ALINP1,EINP1,MTOP1
       COMMON /quama/MQ,MP,TH
       data EINP_old/0.0/,ALIN_old/0.0/,MTOP_old/0.0/,FTQ_old/-1/
C       write(*,*) sqrt(S),ALINP,EINP,MTOP,AL,NFX,FTQ,IOR
C       write(*,*) ' AS',as
C       write(*,*) ' LA',la
       IOR1=IOR
       SK=S
       MTT=2.D0*MTOP
C
C IOR=0 switches off QCD correction i.e. alphas=0.0
C
       IF (IOR.EQ.0) THEN
          DO IQ=1,6
             AS(IQ)=0.D0
             LA(IQ)=.1D0
             RP(IQ)=1.D0
             RM(IQ)=1.D0
             do j=1,6
               RR(IQ,j)=1.D0
             enddo
             MS(IQ)=MQ(IQ)
          ENDDO
       ELSE
C Initialize RG coefficients up to 3rd order for NF=3,4,5,6
C Fill common ASETC: AS(NF),LA(NF),... at different flavor thresholds
       if ((EINP_OLD.EQ.EINP).AND.(ALIN_OLD.EQ.ALINP).and.
     &     (MTOP_old.eq.MTOP).and.(FTQ_old.eq.FTQ)) then
       else
          call inicoe()
          call lambda(ALINP,EINP,MTOP,FTQ)
C          write(*,*) ' AS(3),LA(3)',as(3),la(3)
       endif
       ENDIF
C Initialize s--threshold values
       TT3=(TH(3))**2
       TT4=(TH(4))**2
       TT5=(TH(5))**2
       TT6=MTT**2
       TT(1)=TT3
       TT(2)=TT3
       TT(3)=TT3
       TT(4)=TT4
       TT(5)=TT5
       TT(6)=TT6
       ASQ=0.D0
       RMQ=1.D0
       NF=6
       IF(SK.LT.TT6) NF=5
       IF(SK.LT.TT5) NF=4
       IF(SK.LT.TT4) NF=3
       IF(SK.LT.TT3) then
          write (*,*) ' S.LT.TT3 ?',sqrt(s),sqrt(tt3)
          STOP ' ALQCD2: s outside perturbative region! '
       endif
       IF(IOR.GT.0) THEN
C
C### CALCULATION OF RUNNING PARAMETERS
C
          call getcoe(NF)
          LL=LA(NF)**2
          QQ=SK
          L=LOG(QQ/LL)
          CALL FALP(L,FIQ)
          ASQ=CS0/FIQ
          RMS=RFUN(FIQ)
          RMQ=RMS/RM(NF)*RR(NF,NF)
          RMS2=RMQ**2
       ENDIF
       AL=ASQ
       NFX=NF
       EINP_old=EINP 
       ALIN_old=ALINP
       MTOP_old=MTOP
       FTQ_old =FTQ 
       RETURN
       END
C
       SUBROUTINE M2QCD2(S,ALINP,EINP,MTOP,M2,I,FTQ,IOR)
C      ###################################################
       IMPLICIT NONE
       INTEGER NFX,FTQ,IOR,IQ,NF,I,IOR1,j,icount
       REAL *8 S,ALINP,EINP,MTOP,AL,SK,MTT,XTHR,TT3,TT4,TT5,TT6
     &        ,ASQ,RMQ,LL,QQ,L,FIQ,RMS,RMS2,M2,MA,YT
       REAL *8 EINP_alt,ALIN_alt,MTOP_alt,FTQ_alt,ALINP1,EINP1,MTOP1
       REAL *8 LFUN,RFUN,rpm
       REAL *8 PI,CS0,CS1,CS2,CM0,CM1,CM2
       REAL *8 MQ(6),MP(6),TH(6)
       REAL *8 AS(6),LA(6),RP(6),RM(6),RR(6,6),MS(6),TT(6)
       REAL *8 X0(6),X1(6),X2(6),Y0(6),Y1(6),Y2(6)
       COMMON /COEFF/CS0,CS1,CS2,CM0,CM1,CM2,IOR1
       COMMON /ASETC/ AS,LA,RP,RM,RR,MS,TT
C Input: Quark masses MQ and physical thresholds TH for switching NF
       common /var1/pi,ALINP1,EINP1,MTOP1
       COMMON /quama/MQ,MP,TH
       data icount/0/
       data EINP_alt/0.0/,ALIN_alt/0.0/,MTOP_alt/0.0/,FTQ_alt/-1/
       IOR1=IOR
       SK=S
       MTT=2.D0*MTOP
C
C IOR=0 switches off QCD correction i.e. alphas=0.0
C
       IF (IOR.EQ.0) THEN
          DO IQ=1,6
             AS(IQ)=0.D0
             LA(IQ)=.1D0
             RP(IQ)=1.D0
             RM(IQ)=1.D0
             do j=1,6
               RR(IQ,j)=1.D0
             enddo
             MS(IQ)=MQ(IQ)
          ENDDO
       ELSE
C Initialize RG coefficients up to 3rd order for NF=3,4,5,6
C Fill common ASETC: AS(NF),LA(NF),... at different flavor thresholds
       if ((EINP_alt.eq.EINP).and.(ALIN_alt.eq.ALINP).and.
     &     (MTOP_alt.eq.MTOP).and.(FTQ_alt.eq.FTQ)) then
       else
          call inicoe()
          call lambda(ALINP,EINP,MTOP,FTQ)
C          write(*,*) ' AS(3),LA(3)',as(3),la(3)
       endif
       ENDIF
C Initialize s--threshold values
       TT3=(TH(3))**2
       TT4=(TH(4))**2
       TT5=(TH(5))**2
       TT6=MTT**2
       TT(1)=TT3
       TT(2)=TT3
       TT(3)=TT3
       TT(4)=TT4
       TT(5)=TT5
       TT(6)=TT6
       ASQ=0.D0
       RMQ=1.D0
       NF=6
       IF(SK.LT.TT6) NF=5
       IF(SK.LT.TT5) NF=4
       IF(SK.LT.TT4) NF=3
       IF(SK.LT.TT3) then
          write (*,*) ' S.LT.TT3 ?',sqrt(s),sqrt(tt3)
          STOP ' M2QCD2: s outside perturbative region! '
       endif
       IF(IOR.GT.0) THEN
C
C### CALCULATION OF RUNNING PARAMETERS
C
          call getcoe(NF)
          LL=LA(NF)**2
          QQ=SK
          L=LOG(QQ/LL)
          CALL FALP(L,FIQ)
          ASQ=CS0/FIQ
          RMS=RFUN(FIQ)
          RMQ=RMS/RM(NF)*RR(NF,I)
          RMS2=RMQ**2
       ENDIF
       MA=RR(I,I)*MQ(I)
       YT=4.D0*MA**2/SK
       IF ((FTQ.GT.0).AND.(YT.LT.1D0)) THEN
          MA=RMQ*MQ(I)
       ENDIF
       M2=MA**2
       EINP_alt=EINP 
       ALIN_alt=ALINP
       MTOP_alt=MTOP
       FTQ_alt =FTQ 

       RETURN
       END

      subroutine falp(L,FIQ)
C Running alphas at scale mu in MSbar scheme to three-loop; calculate 
C f=(4pi)/(beta0*alphas(mu)) iteratively from L=ln(mu**2/Lambda**2)
C -> alphas(mu)=(4pi)/(beta0*f)
      implicit none
      integer i,ior 
      real *8 l,fiq,alphas,f,lnx,t1,t8,t13,t14
      real *8 cs0,b1,b2,c0,c1,c2
      real *8 vb2
      common /coeff/cs0,b1,b2,c0,c1,c2,ior
      if (ior.eq.1) then
         FIQ=L
         return
      else 
         F  =L
         DO I=1,6
           FIQ=L+B1*LOG(F+B1)
           F  =FIQ
         ENDDO
         if (ior.eq.2) return 
         t8  = b1**2
         vb2 =4.d0*b2-t8
         t13 = sqrt(abs(vb2))
         t14 = 1.d0/t13
C         f   = L
C         write (*,*) f,vb2,(2.d0*f+b1)*t14
         do i=1,6
           t1 = f**2
           if (vb2.gt.0.d0) then
              FIQ= L+b1*log(t1+b1*f+b2)/2.d0-(t8-2.d0*b2)*t14
     #            *atan((2.d0*f+b1)*t14)
           else if (vb2.eq.0.d0) then
              FIQ= L+b1*log(t1+b1*f+b2)/2.d0-(2.d0*b2-t8)/
     #             (2.d0*f+b1)
           else
              FIQ= L+b1*log(t1+b1*f+b2)/2.d0+(2.d0*b2-t8)*t14
     #            *log((2.d0*f+b1-t13)/(2.d0*f+b1+t13))/2.d0
           endif
           f  = FIQ
         enddo
      endif
      return
      end

      DOUBLE PRECISION function lfun(f)
C Lambda_QCD from alphas at scale mu in MSbar scheme to three-loop
C calculate ln(Lambda**2/mu**2)=-(f(mu)+...); f=(4pi)/(beta0*alphas(mu))
C -> Lambda=mu*lfun(f)
      implicit none
      integer ior
      real *8 alphas,f,lnx,t1,t8,t13,t14
      real *8 cs0,b1,b2,c0,c1,c2
      real *8 vb2
      common /coeff/cs0,b1,b2,c0,c1,c2,ior
C      write (*,*) ' b1,...',b1,b2
      if (ior.eq.3) then
         t1 = f**2
         t8 = b1**2
         vb2=4.d0*b2-t8
         t13 = sqrt(abs(vb2))
         t14 = 1.d0/t13
         if (vb2.gt.0.d0) then
            lnx = f-b1*log(t1+b1*f+b2)/2.d0+(t8-2.d0*b2)*t14
     #           *atan((2.d0*f+b1)*t14)
         else if (vb2.eq.0.d0) then
            lnx = f-b1*log(t1+b1*f+b2)/2.d0-(t8-2.d0*b2)/
     #             (2.d0*f+b1)
         else
            lnx = f-b1*log(t1+b1*f+b2)/2.d0-(2.d0*b2-t8)*t14
     #           *log((2.d0*f+b1-t13)/(2.d0*f+b1+t13))/2.d0
         endif
         lfun=exp(-0.5d0*lnx)
      else if (ior.eq.2) then
         lfun=EXP(-0.5d0*F*(1.D0-B1/F*LOG(F+B1)))
      else
         lfun=EXP(-0.5d0*F)
      endif
      return
      end

      DOUBLE PRECISION function rfun(f)
C Quark mass renormalization factor r(mu) from alphas at scale mu in 
C MSbar scheme to three loops
C calculate r=exp(-2*(gamma0/beta0*lnf(mu)+...)); f=(4pi)/(beta0*alphas(mu))
      implicit none
      integer ior
      real *8 alphas,f,lnx,lnr,t1,t8,t9,t22,t24,t25
      real *8 cs0,b1,b2,c0,c1,c2
      real *8 vb2
      common /coeff/cs0,b1,b2,c0,c1,c2,ior
      if (ior.eq.3) then
         t1 = 1.d0/b2
         t8 = -c0*b2+c2
         t9 =  f**2
         t22 = b1**2
         vb2 =4.d0*b2-t22
         t24 = sqrt(abs(vb2))
         t25 = 1.d0/t24
         if (vb2.gt.0.d0) then
            lnr = -t1*c2*log(f)+t1*(t8*log(t9+b1*f+b2)/2.d0
     #            +2.d0*(-b2*c1+b1*c2-b1*t8/2.d0)*t25
     #            *atan((2.d0*f+b1)*t25))
         else if (vb2.eq.0.d0) then
            lnr = -t1*c2*log(f)+t1*(t8*log(t9+b1*f+b2)/2.d0
     #            -2.d0*(-b2*c1+b1*c2-b1*t8/2.d0)/
     #             (2.d0*f+b1))
         else
            lnr = -t1*c2*log(f)+t1*(t8*log(t9+b1*f+b2)/2.d0
     #            +2.d0*(-b2*c1+b1*c2-b1*t8/2.d0)*t25
     #            *log((2.d0*f+b1-t24)/(2.d0*f+b1+t24))/2.d0)
         endif
         rfun=exp(lnr)
      else if (ior.eq.2) then
         rfun=EXP(-(C0*LOG(F)+C1*LOG(1.D0+B1/F)))
      else
         rfun=EXP(-C0*LOG(F))
      endif
      return
      end

      subroutine inicoe() 
      implicit none
      INTEGER ini,IN,NF,NF2
      REAL *8 PI,ZETA3
      REAL *8 BE0,BE1,BE2,BE02,BE03,GA0,GA1,GA2,CS0,CS1,CS2,CM0,CM1,CM2
      REAL *8 X0(6),X1(6),X2(6),Y0(6),Y1(6),Y2(6)
      COMMON /RGCOE/X0,X1,X2,Y0,Y1,Y2
      save
      data pi /3.141592653589793d0/,zeta3 /1.20205690315959d0/
      data ini/0/
      if (ini.eq.0) goto 2
      return
 2    DO 20 IN=1,4
      NF=IN+2
      NF2=NF**2
      BE0=11.D0-2.D0/3.D0*NF                                              
      BE1=102.D0-38.D0/3.D0*NF                                            
      BE2=2857.D0/2.D0-5033.D0/18.D0*NF+325.D0/54.D0*NF2
      BE02=BE0**2
      BE03=BE02*BE0
C Convention as in Gasser and Leutwyler PR
      GA0=2.D0                                                          
      GA1=101.D0/12.D0-5.D0/18.D0*NF  
C Convention as Gorishny, Kataev and Larin PL 135B (1984) 457 
C                (Result from Tarasov)                                     
      GA2=3747.D0/3.D0+
     &   (160.D0/3.D0*ZETA3-2216.D0/27.D0)*NF-140.D0/81.D0*NF2
      CS0=4.D0*PI/BE0                                                   
      CS1=BE1/BE02                                                   
      CS2=BE2/BE03
      CM0=2.D0*GA0/BE0
      CM1=8.D0*GA1/BE02
      CM2=GA2/BE03
      X0(NF)=CS0
      X1(NF)=CS1
      X2(NF)=CS2
      Y0(NF)=CM0
      Y1(NF)=CM1
      Y2(NF)=CM2
C      write (7,*) ' qcdpar2: nf,beta0,beta1/beta0^2,beta2/beta0^3'
C      write (7,*) nf,be0,cs1,cs2 
  20  CONTINUE
C      write (2,*)  ' '
C      write (2,99) ' CS0: ',X0
C      write (2,99) ' CS1: ',X1
C      write (2,99) ' CS2: ',X2
C      write (2,99) ' CM0: ',Y0
C      write (2,99) ' CM1: ',Y1
C      write (2,99) ' CM2: ',Y2
C      write (2,*)  ' '
      ini=1
      return
   99 format (1x,A6,6(F7.4))
      end

      subroutine getcoe(NF)
      implicit none
      INTEGER NF,ior
      REAL *8 CS0,CS1,CS2,CM0,CM1,CM2
      REAL *8 X0(6),X1(6),X2(6),Y0(6),Y1(6),Y2(6)
      COMMON /RGCOE/X0,X1,X2,Y0,Y1,Y2
      common /coeff/cs0,cs1,cs2,cm0,cm1,cm2,ior
      CS0=X0(NF)
      CS1=X1(NF)
      CS2=X2(NF)
      CM0=Y0(NF)
      CM1=Y1(NF)
      CM2=Y2(NF)
      return
      end

      subroutine lambda(ALIN,EINP,MTOP,FTQ)
C
C### CALCULATION OF THRESHOLD PARAMETERS -> Lambda^(NF)_MSbar
C
       implicit none
       INTEGER FTQ,IOR,NF,NF0,i,j,icount
       REAL *8 ALIN,EINP,MTOP,LA0,LAQ,MTT,MT0,XTHR
       REAL *8 ASQ,LL,QQ,L,FIQ,RMS,ALINP1,EINP1,MTOP1
       REAL *8 LFUN,RFUN,rpm
       REAL *8 PI,CS0,CS1,CS2,CM0,CM1,CM2
       REAL *8 MQ(6),MP(6),TH(6)
       REAL *8 AS(6),LA(6),RP(6),RM(6),RR(6,6),MS(6),TT(6)
       COMMON /COEFF/CS0,CS1,CS2,CM0,CM1,CM2,IOR
       COMMON /ASETC/ AS,LA,RP,RM,RR,MS,TT
C Input: Quark masses MQ and physical thresholds TH for switching NF
       common /var1/pi,ALINP1,EINP1,MTOP1
       COMMON /quama/MQ,MP,TH
       data icount/0/
C Top threshold treatment
       XTHR=1.D0
C       XTHR=0.D0
       IF ((FTQ.EQ.2).OR.(FTQ.EQ.0)) XTHR=0.D0
       MTT=2.D0*MTOP*rpm(XTHR,6,0.1D0,IOR)
       TH(6)=MTT
       NF=6
       IF(EINP.LT.TH(6)) NF=5
       IF(EINP.LT.TH(5)) NF=4
       IF(EINP.LT.TH(4)) NF=3
       IF(EINP.LT.TH(3)) then 
          write (*,*) ' EINP=',EINP,' TH3=',TH(3)
          STOP ' EINPut out of perturbative region! '
       ENDIF
       call getcoe(NF)
       LA0=EINP*lfun(CS0/ALIN)
       NF0=NF
       IF (NF0.EQ.3) GOTO 330
       IF (NF0.EQ.4) GOTO 440
       IF (NF0.EQ.5) GOTO 550
550    NF=5
       LA(5)=LA0
       call getcoe(NF)
       LL=LA(5)**2
       QQ=TH(5)**2
       L=LOG(QQ/LL)
       if (l.lt.0.0d0) write(*,*) ' qcdpar2:nf,ll,qq',nf,ll,qq
       CALL FALP(L,FIQ)
       ASQ=CS0/FIQ
       AS(5)=ASQ
       RMS=RFUN(FIQ)
       RM(5)=RMS
       NF=4
       call getcoe(NF)
       FIQ=CS0/ASQ
       LAQ=TH(5)*LFUN(FIQ)
       LA(4)=LAQ
       RMS=RFUN(FIQ)
       RP(4)=RMS
440    NF=4
       IF(NF0.EQ.4) LA(4)=LA0
       call getcoe(NF)
       LL=LA(4)**2
       QQ=TH(4)**2
       L=LOG(QQ/LL)
       if (l.lt.0.0d0) write(*,*) ' qcdpar2:nf,ll,qq',nf,ll,qq
       CALL FALP(L,FIQ)
       ASQ=CS0/FIQ
       AS(4)=ASQ
       RMS=RFUN(FIQ)
       RM(4)=RMS
       NF=3
       call getcoe(NF)
       FIQ=CS0/ASQ
       LAQ=TH(4)*LFUN(FIQ)
       LA(3)=LAQ
       RMS=RFUN(FIQ)
       RP(3)=RMS
330    NF=3
       IF(NF0.EQ.3) LA(3)=LA0
       call getcoe(NF)
       LL=LA(3)**2
       QQ=TH(3)**2
       L=LOG(QQ/LL)
       if (l.lt.0.0d0) write(*,*) ' qcdpar2:nf,ll,qq',nf,ll,qq
       CALL FALP(L,FIQ)
       ASQ=CS0/FIQ
       AS(3)=ASQ
       RMS=RFUN(FIQ)
       RM(3)=RMS
C
       IF (NF0.EQ.3) GOTO 333
       IF (NF0.EQ.4) GOTO 444
       IF (NF0.EQ.5) GOTO 555
333    NF=3
       call getcoe(NF)
       LL=LA(3)**2
       QQ=TH(4)**2
       L=LOG(QQ/LL)
       CALL FALP(L,FIQ)
       ASQ=CS0/FIQ
       AS(4)=ASQ
       RMS=RFUN(FIQ)
       RP(3)=RMS
       NF=4
       call getcoe(NF)
       FIQ=CS0/ASQ
       LAQ=TH(4)*LFUN(FIQ)
       LA(4)=LAQ
       RMS=RFUN(FIQ)
       RM(4)=RMS
444    NF=4
       call getcoe(NF)
       LL=LA(4)**2
       QQ=TH(5)**2
       L=LOG(QQ/LL)
       CALL FALP(L,FIQ)
       ASQ=CS0/FIQ
       AS(5)=ASQ
       RMS=RFUN(FIQ)
       RP(4)=RMS
       NF=5
       call getcoe(NF)
       FIQ=CS0/ASQ
       LAQ=TH(5)*LFUN(FIQ)
       LA(5)=LAQ
       RMS=RFUN(FIQ)
       RM(5)=RMS
555    NF=5
       call getcoe(NF)
       LL=LA(5)**2
       QQ=TH(6)**2
       L=LOG(QQ/LL)
       CALL FALP(L,FIQ)
       ASQ=CS0/FIQ
       AS(6)=ASQ
       RMS=RFUN(FIQ)
       RP(5)=RMS
       NF=6
       call getcoe(NF)
       FIQ=CS0/ASQ
       LAQ=TH(6)*LFUN(FIQ)
       LA(6)=LAQ
       RMS=RFUN(FIQ)
       RM(6)=RMS
C ************
       do i=1,6
         do j=1,6
           RR(i,j)=1.D0
         enddo
       enddo
       do i=1,3
         RR(4,i)=RR(3,i)*RP(3)/RM(3)
       enddo
       do i=1,4
         RR(5,i)=RR(4,i)*RP(4)/RM(4)
       enddo
       do i=1,5
         RR(6,i)=RR(5,i)*RP(5)/RM(5)
       enddo
       if (icount.eq.0) then
          icount=icount+1
          write (*,*) " Physical thresholds not recalculated"
          write (*,*) " in this version (jan 2003) !"
       endif
c       MTT=MTOP*2.D0*rpm(XTHR,6,AS(6),IOR)
c       MT0 =MTOP
C ************
15     continue
c       MQ(6)=MT0
c       MTT  =2.D0*MQ(6)*rpm(XTHR,6,AS(6),IOR)
c       TH(3)=1.0D0
c       TH(4)=2.D0*MQ(4)*rpm(XTHR,4,AS(4),IOR)
c       TH(5)=2.D0*MQ(5)*rpm(XTHR,5,AS(5),IOR)
c       TH(6)=MTT
C ************
       return
       end

       DOUBLE PRECISION function arctanh(x)
       implicit none
       real *8 x
       if (x**2.ge.1.d0) then
          stop ' arctanh(x) not defined for x**2 >= 1. '
       endif
       arctanh=0.5d0*log((1.d0+x)/(1.d0-x))
       return
       end

       function rpm(XTHR,NF,ASQ,IOR)
       implicit none
       integer NF,IOR,i 
       real *8 pi,rpm,XTHR,ASQ,M,adp,c,ALINP1,EINP1,MTOP1
       REAL *8 MQ(6),MP(6),TH(6)
       common /var1/pi,ALINP1,EINP1,MTOP1
       COMMON /quama/MQ,MP,TH
       adp=asq/pi
       rpm=1.d0
       m  =mq(nf)
       if (ior.eq.2) then  
         rpm=1.d0+xthr*4.d0/3.d0*adp
       else if (ior.eq.3) then
         c=0.d0
         do i=1,NF-1
           c=c+1.d0-mq(i)/m
         enddo
         rpm=1.d0+xthr*(4.d0/3.d0*adp+(16.11d0-1.04d0*c)*adp**2)
       else
       endif      
       return
       end

      subroutine getnf(e,nf)
C find n_f effective for given energy scale e
      implicit none
      integer nf,i,j
      real *8 e
      real *8 MQ(6),MP(6),TH(6)
      COMMON /quama/MQ,MP,TH
      if(e.lt.0.999999d0)
     &     stop ' energy scale out of perturbative range! '
      nf=6
      do i=1,3
        if(e.lt.th(nf)) nf=nf-1
      enddo
      return
      end

C% = -B1*LOG(F**2+B1*F+B2)/2.0+(B1**2-2*B2)*ATAN((2*F+B1)/SQRT(4*B2
C 1          -B1**2))/SQRT(4*B2-B1**2)+F
C      D5 = (CX2-B2*CX0)*LOG(F**2+B1*F+B2)/B2/2.0+(B1*CX2-2*B2*CX1+B1*B2*
C     1   CX0)*ATAN((2*F+B1)/SQRT(4*B2-B1**2))/(B2*SQRT(4*B2-B1**2))-CX2*
C     2   LOG(F)/B2


       subroutine SETQM1(mpol,ALINP,EINP,MTOP,I,FTQ,IOR)
C Update the value of the running MS-bar quark mass MQ(I) in 
C common /quama/ given the pole mass mpol=mq=mq(mq)
       implicit none
       integer I,FTQ,IOR,NF 
       real *8 mpol,ALINP,EINP,MTOP,AL,rpm,XTHR,XX
C      REAL *8 MQ(6),TH(6)
       real *8 MQ(6),MP(6),TH(6)
       REAL *8 AS(6),LA(6),RP(6),RM(6),RR(6,6),MS(6),TT(6)
       COMMON /ASETC/ AS,LA,RP,RM,RR,MS,TT
       COMMON /quama/MQ,MP,TH
       call ALQCD2(TH(I)**2,ALINP,EINP,MTOP,AL,NF,FTQ,IOR)
       XTHR=1.d0
       XX=RR(I,I)*rpm(XTHR,I,AS(I),IOR)
       IF (XX.GT.0.D0) THEN
         MQ(I)=MPOL/XX
         WRITE (*,99) I,MQ(I)
       ELSE
         WRITE (*,*) ' MQ(',i,') NOT UPDATED !  not positive ! '
       ENDIF
       return
 99    format(1x,' MQ(',i1,') UPDATED ! ',f7.3)
       end
