      subroutine fcn(x,y,f)
*
* BF-MOM scheme
* external for NAG D02BDF integration routine
* input x, f(x), output f=dy/dx(x)
*
      implicit none
      integer n
      parameter(n=1)
      real *8 x,y(n),f(n)
      integer ior,i,j,nH,nL,nF
      real *8 hmom,b0x,b1x,t0x,t1x,eex,mmx
      real *8 sum5,sum6,beta0,beta1
      real *8 CF,CA,TF
      REAL *8 mm(6)
C Input: Quark masses MQ and physical thresholds TH for switching NF
      common /case/ior
      common /group/CF,CA,TF,nL
      COMMON /pmass/mm

      sum5=0.0d0
      sum6=0.0d0
*   implemented for Euclidean region only 
* (i.e. space like momentum transfer assumed)
      eex =-abs(x)
      do i=1,3
          mmx=mm(i+3)          
          call funb(b0x,b1x,eex,mmx) 
C          call funa(b0x,b1x,eex,mmx)   ! rational approximation for b1x 
C          call btes(t0x,t1x,eex,mmx)
C          write (*,*) ' b1x,t1x:',b1x,t1x
* sums for beta--functions eq. (15)
         sum5=sum5+b0x               
         sum6=sum6+b1x               
      enddo
      beta0=0.d0
      beta1=0.d0
      if (ior.ge.1) then
        beta0=11.d0/3.d0*CA-4.d0/3.d0*TF*(nL+sum5)
      else
      endif
      if (ior.ge.2) then
        beta1=34.d0/3.d0*CA**2-4.d0*(5.d0/3.d0*CA+CF)*TF*nL-TF*sum6
      else
      endif
C      beta0=11.d0/3.d0*CA-4.d0/3.d0*TF*(nL+2)
C      beta1=34.d0/3.d0*CA**2-4.d0*(5.d0/3.d0*CA+CF)*TF*(nL+2)
* RG eq. (14)
      hmom=y(1)
C      write (*,*) ' beta0,beta1=',beta0,beta1
      f(1)=-(beta0+beta1*hmom)*hmom**2/x
      return
      end

      function varylog(i,y)
      implicit none
      integer i
      real *8 y,varylog,ddilog,dli3
      if (i.eq.2) then
        varylog=ddilog(y)
C        write(3,*) ' li2:',varylog
      else if (i.eq.3) then
        varylog=dli3(y)
C        write(3,*) ' li3:',varylog
      else
        stop ' varylog not defined'
      endif
      return
      end

      subroutine funb(b0x,b1x,eex,mmx)
      implicit none
      integer ior,nL
      real *8 b0x,b1x,eex,mmx
      real *8 CF,CA,TF,x,x2,x3,z,y,Gvz,Ivz,I3z,sig,varylog
      REAL *8 PI,Zeta2,Zeta3
      common /case/ior
      common /group/CF,CA,TF,nL
      common /const/pi,zeta2,zeta3

      z=eex/4.d0/mmx
      y=(sqrt(1.d0-1.d0/z)-1.d0)/(sqrt(1.d0-1.d0/z)+1.d0)
      x =z
      x2=x**2
      x3=x2*x
*
      b0x=0.d0
      b1x=0.d0
      if (ior.ge.1) then
        Gvz=(2.d0*y*log(y)/(y**2-1.d0))

*   this is ONE-LOOP correction
*   formula (16a) from Jegerlehner Tarasov paper

        b0x=1.d0+3.d0/(2.d0*x)*(1.d0-Gvz)
      else
      endif
      if (ior.ge.2) then 
        Ivz=(6.d0*(Zeta3+4.d0*varylog(3,-y)+2.d0*varylog(3,y))
     &      -8.d0*(2.d0*varylog(2,-y)+varylog(2,y))*log(y)
     &      -2.d0*(2.d0*log(1.d0+y)+log(1.d0-y))*log(y)**2
     &      )
        I3z=(6.d0*Zeta3-6.d0*varylog(3,y)+6.d0*log(y)*varylog(2,y)
     &      +2.d0*log(1.d0-y)*log(y)**2)
        sig=((1.d0-y**2)/y*(2.d0*varylog(2,-y)+varylog(2,y)
     &      +(log(1.d0-y)+2.d0*log(1.d0+y)-3.d0/4.d0*log(y))*log(y))
     &      )

*   this is TWO-LOOP correction
*   formula (16b) from JT paper

        b1x=-(16.d0*(1.d0-x2)*CF+(1.d0+8.d0*x2)*CA)
     &           *sig/(6.d0*x2*(1.d0-x))
     &     +2.d0/(3.d0*x2)*(CA-2.d0*CF)*Ivz 
     &     -2.d0/(3.d0*x)*I3z*CA
     &     -((1.d0+3.d0*x-10.d0*x2+12.d0*x3)*CA
     &     -3.d0*(3.d0-3.d0*x-4.d0*x2+8.d0*x3)*CF)*4.d0/(3.d0*x)*Gvz**2
     &     +((147.d0-4.d0*x-100.d0*x2+8.d0*x3)*CA+168.d0*(1.d0-x)*CF 
     &     + 6.d0*(9.d0+4.d0*x)*log(-4.d0*x)*CA)/(9.d0*x)*Gvz
     &     -((99.d0+62.d0*x)*CA+12.d0*(11.d0+3.d0*x)*CF
     &     +2.d0*(27.d0+24.d0*x-2.d0*x2)*log(-4.d0*x)*CA)/(9.d0*x)
        b1x=-b1x
      else
      endif
      return
      end

      subroutine btes(t0x,t1x,eex,mmx)
      implicit none
      integer ior,nL
      real *8 t0x,t1x,eex,mmx
      real *8 CF,CA,TF,r,B1
      REAL *8 PI,Zeta2,Zeta3
      common /case/ior
      common /group/CF,CA,TF,nL
      common /const/pi,zeta2,zeta3

      r=mmx/abs(eex)
*
      t0x=0.d0
      t1x=0.d0
      if (ior.ge.1) then
        t0x=1.d0
      else
      endif
      if (ior.ge.2) then 
        B1=(-0.45577d0*r+0.26995d0)/(r**2+2.1742d0*r+0.26995d0)
        t1x=B1*4.d0/3.d0*(5.d0*CA+3.d0*CF)
      else
      endif
      return
      end

      subroutine funa(b0x,b1x,eex,mmx)
      implicit none
      integer ior,nL
      real *8 b0x,b1x,eex,mmx,z,y,Gvz
      real *8 CF,CA,TF,r,B1
      REAL *8 PI,Zeta2,Zeta3
      common /case/ior
      common /group/CF,CA,TF,nL
      common /const/pi,zeta2,zeta3

      r=mmx/abs(eex)
*
      b0x=0.d0
      b1x=0.d0
      if (ior.ge.1) then
        z=eex/4.d0/mmx
        y=(sqrt(1.d0-1.d0/z)-1.d0)/(sqrt(1.d0-1.d0/z)+1.d0)
        Gvz=(2.d0*y*log(y)/(y**2-1.d0))
        b0x=1.d0+3.d0/(2.d0*z)*(1.d0-Gvz)
      else
      endif
      if (ior.ge.2) then 
        B1=-0.4960247030d0*(r-0.6430100000d0)/
     &    (r**2+3.623488767d0*r+0.3189488443d0)
        b1x=B1*4.d0/3.d0*(5.d0*CA+3.d0*CF)
      else
      endif
      return
      end

      subroutine gcn(x,y,f)
*
* MSbar scheme, Marciano matching
* external for NAG D02BDF integration routine
* input x, f(x), output f=dy/dx(x)
*
      implicit none
      integer n
      parameter(n=1)
      real *8 x,y(n),f(n)
      integer ior,nf,nf2,nf3
      real *8 a,eex
      real *8 beta0,beta1,beta2,beta3
      common /case/ior

*   implemented for Euclidean region only 
* (i.e. space like momentum transfer assumed)
      eex =abs(x)
* MSbar RG
      beta0=0.d0
      beta1=0.d0
      beta2=0.d0
      beta3=0.d0
      call getnf(sqrt(eex),nf)
      nf2=nf*nf
      nf3=nf2*nf
      if (ior.ge.1) then
        beta0 =  (11.d0-2.d0/3.d0*nf)/4.d0
      else
      endif
      if (ior.ge.2) then
        beta1 =  (102.d0-38.d0/3.d0*nf)/16.d0
      else
      endif
      if (ior.ge.3) then
        beta2 =  (2857.d0/2.d0-5033.d0/18.d0*nf+325.d0/54.d0*nf2)/64.d0
      else
      endif
      if (ior.ge.4) then
        beta3 = 114.23033d0-27.133944d0 * nf + 1.5823791d0 * nf2 
     &      + 0.0058566958d0 * nf3
      else
      endif
* MSbar RG 
      a=y(1)
      f(1)=-(beta0+beta1*a+beta2*a**2+beta3*a**3)*a**2/x
C      write (7,*) ior,beta0,beta1,beta2,beta3
      return
      end

C      subroutine gcn(x,y,f)
C*
C* external for NAG D02BDF integration routine
C* input x, f(x), output f=dy/dx(x)
C*
C      implicit none
C      integer n
C      parameter(n=1)
C      real *8 x,y(n),f(n)
C      integer ior,i,j,nH,nL,nF
C      real *8 hmom,eex,mmx
C      real *8 beta0,beta1
C      real *8 CF,CA,TF
C      common /case/ior
C      common /group/CF,CA,TF,nL
C
C*   implemented for Euclidean region only 
C* (i.e. space like momentum transfer assumed)
C      eex =-abs(x)
C* MSbar RG
C      beta0=0.d0
C      beta1=0.d0
C      if (ior.ge.1) then
C        beta0=11.d0/3.d0*CA-4.d0/3.d0*TF*(nL+3)
C      else
C      endif
C      if (ior.ge.2) then
C        beta1=34.d0/3.d0*CA**2-4.d0*(5.d0/3.d0*CA+CF)*TF*(nL+3)
C      else
C      endif
C* MSbar RG 
C      hmom=y(1)
CC      write (*,*) ' beta0,beta1=',beta0,beta1
C      f(1)=-(beta0+beta1*hmom)*hmom**2/x
C      return
C      end

      subroutine getnf(e,nf)
C find n_f effective for given energy scale e
      implicit none
      integer nf,i,j
      real *8 e
      real *8 mq(6),mp(6),th(6)
      common /quark/mq,mp,th
      if(e.lt.0.9999d0) stop ' energy scale out of perturbative range! '
      nf=6
      do i=1,3
        if(e.lt.th(nf)) nf=nf-1
      enddo
      return
      end
