      Program qcdpar4_test
C QCD  PARAMETERS: TEST PROGRAM for qcdpar4.f
* Numerical integration of RG eq. using NAG routine D02BDF 
*             (Runge-Kutta-Merson method)
C This program calculates BF-MOM and MSb evolution of QCD coupling 
C by numerical integration. BF-MOM 1,2-loop; MSb 1,2,3,4-loop with BW matching.
* This program takes the same (MSb-)input "alinp" at "einp" for BF-MOM and MSb.
* The proper BF-MOM input alinp(MOM)at einp can be obtained with the qcdpar3.f
* routine which uses analytic form by Shirkov. It should be used in conjunction
* with a rescaling factor alpha_sMOM(x Q^2) which absorbs to large constant
* terms in the MOM<->MSb relationship.
C Linkfile : qcdpar4,li3,li2,...
C Input: order of pQCD: ior=1,2,3,4 (read from screen)
C Output: file qcdpar4_alsi.ou"ior"  
C Option: Proper BF-MOM with rescaling by "xsc" 
      IMPLICIT NONE
      character *1 cfn(4)
      integer n,ifail,ifun,i,nout
      parameter(n=1,nout=6)
      INTEGER IP,NP,II,K,NF0,NFS,FTQ,IOR,nH,nL,nF,nfx,irsc
      REAL *8 ALINP,EINP,EMIN,EMAX,SMIN,SMAX,LOSU,LOSL,DLST,LSLT
     &       ,S,E,ALFA,ALFAMOM,alfamom_old,y_old,s_old,ftes
     &       ,alfamsb,alfatau1,alfatau2,alfamsb_old,z_old,gtes,alerr
      real *8 tol(n),ymax(n),y(n),f(n),z(n),zz(n),zy(n),zx(n),
     . g(n),t(n),tz(n),ty(n),tx(n),u(n),eps,
     &     yin(n),you(n),tin(n),tou(n),uin(n),uou(n),zin(n),zou(n)
      real *8 CF,CA,TF
      REAL *8 pi,zeta2,zeta3,fourpi
      real *8 x,xend,xfin,ax,anfminus,lnx,xsc,lsc
      real *8 mm(6),mtop
      external fcn,gcn,D02BDF
      common /case/ior
      common /group/CF,CA,TF,nL
      common /const/pi,zeta2,zeta3
      COMMON /pmass/mm
      data zeta2,zeta3/1.644934066848226d0,1.20205690315959d0/
      data cfn /'1','2','3','4'/
C Input: Quark masses MQ and physical thresholds TH for switching NF from rqcd-param.f
      integer IER
      double precision pi1,ALINP1,EINP1,MTOP1,ERALS
      DOUBLE PRECISION MQ(6),MP(6),TH(6),dmp(6,3)
      common /inp/pi1,ALINP1,EINP1,MTOP1
      COMMON /QUARK/MQ,MP,TH
      common /QUAER/ERALS,IER
      common /QUAME/DMP

      call rqcdparam()          ! fill common inp,QUARK etc; updated April 2008 and 2012
c now as interface to constants_qcd
      pi   =pi1
      alinp=ALINP1
      einp =EINP1
      alerr=ERALS
*
*     .. Executable Statements ..
*       
      fourpi=4.d0*pi
*  Initial data for the GAUGE GROUP

      CF=4.d0/3.d0
      CA=3.d0
      TF=1.d0/2.d0

* nL is the number of massless quarks
* nH is the number of massive quarks

      nH=3
      nL=3
      nF=nL+nH

*################################################

*  momentum, masses and normalization point:

C      alf0=0.12d0
C      mu2=(91.19d0)**2

*     heavy quark masses (here pole masses)
* test
C       do i=1,2
C         mq(i+3)=0.5d0
C         mp(i+3)=0.5d0
C         th(i+3)=1.0d0
C       enddo
* test

      mm(4)=mp(4)**2
      mm(5)=mp(5)**2
      mm(6)=mp(6)**2

*#################################################
*#################################################
*#################################################
C
c  7    FORMAT(1X ,2X,' Strong coupling ALINP at EINP :'/
c     1 ,3X,' Test-range: EMIN,EMAX,NP (Anzahl Punkte)'/
c     1 ,3X,' # IOR=1,2 order of perturbation theory '/
c     1 ,3X,' # enter ALINP,EINP,EMIN,EMAX,NP,IOR'/)
c       WRITE(*,7)
C       READ(*,*) ALINP,EINP,EMIN,EMAX,NP,IOR
* this program takes the same MSb input alinp at einp for BF-MOM and MSb
C Option: irsc=0: MSb input for both BF-MOM and MSb
C         irsc=1: BF-MOM input with energies rescaled to e -> lsc*e
       xsc=1.d0        ! quadratic rescaling factor for energy squares
       lsc=1.d0        ! linear rescaling factor for energies
* specify energy range here
       emin=1.d0
       emax=100.d0
       irsc=1          ! MSb:0 versus MOM:1 input
       ior=2
       WRITE(*,*) ' IOR=1,2,3,4(loops),irsc=0,1(rescale)?'
       READ(*,*) IOR,irsc
       if (irsc.eq.1) then
         if (ior.eq.1) then
           alinp=alinp+alerr      
         else if (ior.eq.2) then
           alinp=alinp-alerr   
         else
         endif
       else
       endif
* MSb -> BF-MOM at MZ: aminp=.137d0
       if (irsc.eq.1) then
* rescaling factor alpha_sMOM (x Q^2) x such that
* -> alpha_sMSb ( Q^2) for Q^2 -> infty
* lnx=(205.d0/36.d0*CA-20.d0/9.d0*TF*nF)/(11.d0/3.d0*CA-4.d0/3.d0*TF*nF)
         lnx=125.d0/12.d0/7.d0
         xsc=exp(lnx)
         lsc=sqrt(xsc)
         write (*,*) ' scale factor lsc:2.1044...?',lsc
* Alternatives: use alinp=.137d0 at einp=91.19d0 as input or
*      or           alinp=.121d0 at einp=lsc*einp  
C         alinp=.137d0       ! get this value with qcdpar3.f
C          alinp=.121d0
          alinp=0.1068209052d0  ! alphaMSb at x_0 M_Z =  191.895 GeV
          einp=lsc*einp
       else
       endif
       emin=emin*lsc
       emax=emax*lsc
C since in the MOM scheme mu is a momentum subtraction point and the 
C location of the thresholds should not change under rescaling we have
C to rescale the masses as well; in fact it means mu in the MSb scheme
C has to be rescaled to lower values mu_MSb -> mu_MSb/lsc 
       do i=4,6
         mm(i)=mm(i)*xsc
       enddo
       np=100
       if (irsc.eq.1) then
         OPEN(UNIT=1,FILE='qcdpar4_alsi.mo'//cfn(ior),STATUS='NEW')
       else
         OPEN(UNIT=1,FILE='qcdpar4_alsi.ou'//cfn(ior),STATUS='NEW')
       endif
       if ((ior.lt.1).or.(ior.gt.2)) then
         write (*,*) ' ior=',ior,' not implemented for
     &         BF_MOM using ior=2!'
c         stop
       endif
C
  8    FORMAT (2X,'ALINP:',F6.3,2X,'EINP:',F6.2,'  IOR:',I1)
  9    FORMAT (1X ,F10.2,3(2X,F9.4))
 10    FORMAT (1X ,F10.2,2(2X,F9.4))
 11    FORMAT (2X,' m_c:',F6.2,2X,' m_b:',F6.2,2X,' m_t:',F8.2)
 12    FORMAT (1X ,F10.2,1(2X,F9.5),2(1X,E11.4),5x,i1)
       WRITE (1,*) '      6.00000'
       WRITE (1,*) '      1.00000'
       WRITE (1,*) ' ',FLOAT(NP)
       WRITE (1,*) ' Results obtained running qcdpar4'
       WRITE(1,8) ALINP,EINP,IOR
       WRITE(1,11) sqrt(mm(4)),sqrt(mm(5)),sqrt(mm(6))
C       WRITE (1,*) ' '
C       WRITE (1,*) ' E,ALFA,NF'
       WRITE (1,*) ' Log(E^2),1./ALFA_MOM,1.d0/ALFA_MSb',
     .  '(numerical integration)'
C       WRITE (1,*) ' '
       WRITE (2,*) ' qcdpar4: E,ALFA_MOM,f(1),ftes,ior'
       WRITE (3,*) ' qcdpar4: E,ALFA_MSb,g(1),gtes,ior'
*
*  Set initial conditions and input for D02PVF
c energy loop set up
       SMIN=EMIN**2
       SMAX=EMAX**2
       LOSU=DLOG(SMAX)
       LOSL=DLOG(SMIN)
       DLST=(LOSU-LOSL)/(NP-1)
       eps=1.d-6
c energy loop
       DO 100 IP=1,NP
         alfamom_old=alfamom
         alfamsb_old=alfamsb
         y_old=y(1)
         z_old=z(1)
         s_old=s
         LSLT=LOSL+(IP-1)*DLST
         S=DEXP(LSLT)
         E=DSQRT(S)
*
         ifun=1   ! fcn for BG-MOM
         eps=1.d-6
         x=EINP**2
         xend=s
         yin(1)=ALINP/fourpi
         ifail=1
*
         call D02BDF(x,xend,xfin,n,yin,y,tol,ifun,ymax,ifail)
c(x,xend,n,y,tol,ifun,ymax,ifail)

         x=EINP**2
         xend=s+(s-s_old)*eps
         yin(1)=ALINP/fourpi
         ifail=1
*
         call D02BDF(x,xend,xfin,n,yin,t,tol,ifun,ymax,ifail)
c(x,xend,n,t,tol,ifun,ymax,ifail)

* Here MSbar RG integration in BW scheme

         e=sqrt(s)
         ifun=2                 ! gcn for MSbar (Marciano matching)
* above Upsilon below top
         if ((th(5).le.e).and.(e.le.th(6))) then
           x=EINP**2
           xend=s
           yin(1)=ALINP/pi
           ifail=1

           call D02BDF(x,xend,xfin,n,yin,z,tol,ifun,ymax,ifail)
c(x,xend,n,z,tol,ifun,ymax,ifail)

           x=EINP**2
           xend=s+(s-s_old)*eps
           yin(1)=ALINP/pi
           ifail=1

           call D02BDF(x,xend,xfin,n,yin,u,tol,ifun,ymax,ifail)
c(x,xend,n,u,tol,ifun,ymax,ifail)
* special value at Upsi--mass 11. GeV
           x=EINP**2
           xend=(11.d0)**2
           yin(1)=ALINP/pi
           ifail=1

           call D02BDF(x,xend,xfin,n,yin,zx,tol,ifun,ymax,ifail)
c(x,xend,n,zx,tol,ifun,ymax,ifail)
* above J/psi below Upsilon
         else if ((th(4).le.e).and.(e.le.th(5))) then
           nfx=5
           x=EINP**2
           xend=th(nfx)**2
           yin(1)=ALINP/pi
           ifail=1

           call D02BDF(x,xend,xfin,n,yin,z,tol,ifun,ymax,ifail)
c(x,xend,n,z,tol,ifun,ymax,ifail)

           x=th(nfx)**2
           xend=s
           ax=anfminus(z(1),nfx,ior)
           yin(1)=ax
           ifail=1

           call D02BDF(x,xend,xfin,n,yin,z,tol,ifun,ymax,ifail)
c(x,xend,n,z,tol,ifun,ymax,ifail)

           x=th(nfx)**2
           xend=s+(s-s_old)*eps
           yin(1)=ax
           ifail=1

           call D02BDF(x,xend,xfin,n,yin,u,tol,ifun,ymax,ifail)
c(x,xend,n,u,tol,ifun,ymax,ifail)
* special value at J/psi--mass 3.3 GeV
           x=th(nfx)**2
           xend=(3.3d0)**2
           yin(1)=ax
           ifail=1

           call D02BDF(x,xend,xfin,n,yin,zy,tol,ifun,ymax,ifail)
c(x,xend,n,zy,tol,ifun,ymax,ifail)
* below J/psi
         else if ((th(3).le.e).and.(e.le.th(4))) then
           nfx=5
           x=EINP**2
           xend=th(nfx)**2
           yin(1)=ALINP/pi
           ifail=1

           call D02BDF(x,xend,xfin,n,yin,z,tol,ifun,ymax,ifail)
c(x,xend,n,z,tol,ifun,ymax,ifail)

           x=th(nfx)**2
           xend=th(nfx-1)**2
           ax=anfminus(z(1),nfx,ior)
           yin(1)=ax
           ifail=1

           call D02BDF(x,xend,xfin,n,yin,z,tol,ifun,ymax,ifail)
c(x,xend,n,z,tol,ifun,ymax,ifail)

           nfx=4
           x=th(nfx)**2
           xend=s
           ax=anfminus(z(1),nfx,ior)
           yin(1)=ax
           ifail=1

           call D02BDF(x,xend,xfin,n,yin,z,tol,ifun,ymax,ifail)
c(x,xend,n,z,tol,ifun,ymax,ifail)

           x=th(nfx)**2
           xend=s+(s-s_old)*eps
           yin(1)=ax
           ifail=1
           
           call D02BDF(x,xend,xfin,n,yin,u,tol,ifun,ymax,ifail)
c(x,xend,n,u,tol,ifun,ymax,ifail)
* special value at tau--mass 1.777 GeV
           x=th(nfx)**2
           xend=(1.777d0)**2
           yin(1)=ax
           ifail=1

           call D02BDF(x,xend,xfin,n,yin,zz,tol,ifun,ymax,ifail)
c(x,xend,n,zz,tol,ifun,ymax,ifail)
        else
        endif         

C         if (ifail.ne.0) then
C           write (nout,99998) ' D02BDF fails. IFAIL =', IFAIL
C         else
C           call out(x,yin,w,stiff)
C         endif
         alfamom=fourpi*y(1)
         alfamsb= pi*z(1)
         WRITE (1,10) log((e/lsc)**2),1.d0/ALFAmom,1.d0/ALFAMSb
* Check derivative
         call fcn(s,yin,f)
         call gcn(s,z,g)
         if (s.ne.s_old) then
           ftes=(t(1)-y(1))/(s-s_old)/eps
           gtes=(u(1)-z(1))/(s-s_old)/eps
         else 
         endif
         if (i.gt.1) WRITE (2,12) E/lsc,ALFAmom,f(1),ftes,ior
         if (i.gt.1) WRITE (3,12) E/lsc,ALFAmsb,g(1),gtes,ior
100    CONTINUE
       ifun=1
* special value at tau--mass
         x=EINP**2
         xend=(lsc*1.777d0)**2
         yin(1)=ALINP/fourpi
         ifail=1
*
         call D02BDF(x,xend,xfin,n,yin,tz,tol,ifun,ymax,ifail)
c(x,xend,n,tz,tol,ifun,ymax,ifail)
* special value at J/Psi--mass
         x=EINP**2
         xend=(lsc*3.3d0)**2
         yin(1)=ALINP/fourpi
         ifail=1
*
         call D02BDF(x,xend,xfin,n,yin,ty,tol,ifun,ymax,ifail)
c(x,xend,n,ty,tol,ifun,ymax,ifail)
* special value at Upsi--mass
         x=EINP**2
         xend=(lsc*11.d0)**2
         yin(1)=ALINP/fourpi
         ifail=1
*
         call D02BDF(x,xend,xfin,n,yin,tx,tol,ifun,ymax,ifail)
c(x,xend,n,tx,tol,ifun,ymax,ifail)
*
       write (1,*) ' tau:alfamom, alfamsb:',fourpi*tz(1),pi*zz(1) 
       write (1,*) ' psi:alfamom, alfamsb:',fourpi*ty(1),pi*zy(1) 
       write (1,*) ' ups:alfamom, alfamsb:',fourpi*tx(1),pi*zx(1) 
       write (1,*) ' normalization: alfamom(EINP)=ALINP '
       write (1,*) ' alfamsb using BW matching '         
       STOP
       STOP
*
99999  format (1X,A,e9.1)
99998  format (1X,A,I3)
       END

       function anfminus(a,nf,ior)
       implicit none
       integer nf,ior
       real *8 a,anfminus
       if (ior.eq.4) then
         anfminus=a*(1.d0+0.1528*a**2+(0.9721-0.0847*(nf-1))*a**3)
       else if (ior.eq.3) then
         anfminus=a*(1.d0+0.1528*a**2)
       else
         anfminus=a
       endif        
       return
       end

      subroutine D02BDF(x,xend,xfin,nq,y,t,tol,ifun,ymax,ifail)
*
* Numerical integration of RG eq. using NAG routine D02PCF 
*             (Runge-Kutta-Merson method)
*
      IMPLICIT NONE
*     .. Parameters ..
      INTEGER          NOUT
      PARAMETER        (NOUT=6)
      INTEGER          NEQ, N, LENWRK, IW, METHOD
      PARAMETER        (NEQ=1,LENWRK=32*NEQ,METHOD=1,N=NEQ,IW=LENWRK)
      DOUBLE PRECISION ZERO, ONE, TWO
      PARAMETER        (ZERO=0.0D0,ONE=1.0D0,TWO=2.0D0)
*     .. Local Scalars ..
      DOUBLE PRECISION HNEXT, HSTART, TEND, TGOT, TINC, TOL, TSTART,
     +                 TWANT, WASTE
      INTEGER          I, IFAIL, J, L, NPTS, STPCST, STPSOK, TOTF
      LOGICAL          ERRASS
*     .. Local Arrays ..
      DOUBLE PRECISION THRES(NEQ), WORK(LENWRK), YGOT(NEQ), YMAX(NEQ),
     +                 YPGOT(NEQ), YSTART(NEQ)
*     .. External Functions ..
      DOUBLE PRECISION X01AAF
*
      integer ifun,nq
      double precision x,xend,xfin,y(nq),t(nq)
*      
      EXTERNAL         X01AAF
*     .. External Subroutines ..
      EXTERNAL         D02PCF, D02PVF, D02PYF, FCN, GCN
*
      TSTART = x
      YSTART(1) = y(1)
c      YSTART(2) = y(2)
      TEND = xend
      DO 20 L = 1, NEQ
         THRES(L) = 1.0D-8
   20 CONTINUE
      ERRASS = .FALSE.
      HSTART = ZERO
*
*  Set control for output
*
      NPTS = 8
      TINC = (TEND-TSTART)/NPTS
*
      DO 60 I = 1, 2
         IF (I.EQ.1) TOL = 1.0D-3
         IF (I.EQ.2) TOL = 1.0D-4
         IFAIL = 0
         CALL D02PVF(NEQ,TSTART,YSTART,TEND,TOL,THRES,METHOD,
     +               'Usual Task',ERRASS,HSTART,WORK,LENWRK,IFAIL)
*
         WRITE (NOUT,'(/A,E8.1)') ' Calculation with TOL = ', TOL
         WRITE (NOUT,'(/A/)') '    sqrt(t)         y'
         WRITE (NOUT,'(1X,F9.4,1(3X,F7.5))')
     &        sqrt(TSTART), (YSTART(L),L=1,NEQ)
         DO 40 J = NPTS - 1, 0, -1
            TWANT = TEND - J*TINC
            IFAIL = 1
            if (ifun.eq.1)
     &           CALL D02PCF(FCN,TWANT,TGOT,YGOT,YPGOT,YMAX,WORK,IFAIL)
            if (ifun.eq.2)
     &           CALL D02PCF(GCN,TWANT,TGOT,YGOT,YPGOT,YMAX,WORK,IFAIL)
*
            WRITE (NOUT,'(1X,F9.4,1(3X,F7.5))')
     &           sqrt(TGOT), (YGOT(L),L=1,NEQ)
   40    CONTINUE
*
         IFAIL = 0
         CALL D02PYF(TOTF,STPCST,WASTE,STPSOK,HNEXT,IFAIL)
         WRITE (NOUT,'(/A,I6)')
     +     ' Cost of the integration in evaluations of F is', TOTF
*         
         t(1)=ygot(1)
         xfin=TGOT
*
   60 CONTINUE
      RETURN
      END
